/* -*- Mode: C; tab-width: 4 -*-
   ylib.h -- Y window system client library
   Created: Chris Toshok <toshok@hungry.com>, 8-May-1997
 */

#ifndef _y_lib_h
#define _y_lib_h

/* types for both the server and client side of things.. */
#include "yfonts.h"
#include "ytypes.h"
#include "ymessages.h"

/* These are defined in ytypes.h
   ---
   typedef struct YWindow_struct YWindow;
   typedef struct YGC_struct YGC;
   typedef struct YFont_struct YFont;
*/
typedef struct YDisplay_struct YDisplay;

extern YDisplay *YOpenDisplay(char *display_name);
extern void YCloseDisplay(YDisplay *display);

extern int YGetDisplayFD(YDisplay *display);

extern YWindow *YCreateWindow(YDisplay *display, YWindow *parent_window,
							  int x, int y, int width, int height,
							  YColor bg_color);

extern YImage *YCreateImage(YDisplay *display,
							int width, int height,
							YColor bg_color);
extern YImage *YCreateImageFromPNGFile(YDisplay *display,
									   char *filename);

extern void YDestroyWindow(YWindow *window);
extern void YDestroyImage(YImage *image);

extern YRect *YGetWindowBounds(YWindow *window);

extern void YMapWindow(YWindow *window);
extern void YUnmapWindow(YWindow *window);
extern void YMoveWindow(YWindow *window, int x, int y);
extern void YResizeWindow(YWindow *window, int width, int height);

extern void YRaiseWindow(YWindow *window);
extern void YLowerWindow(YWindow *window);

extern void YReparentWindow(YWindow *window, YWindow *new_parent);

extern YGC *YGetGC(YWindow *window);
extern YGC *YGetGCById(YGC_id id);
extern void YDestroyGC(YGC *gc);

extern void YSetForeground(YGC *gc, YColor foreground_color);
extern void YSetBackground(YGC *gc, YColor background_color);
extern void YSetFont(YGC *gc, YFont *font);

/* color stuff. */
extern YColor YGetColorRGB(uint8 red, uint8 green, uint8 blue);
extern YColor YGetColorHSV(float h, float s, float v);
extern YColor YGetColorHLS(float h, float l, float s);
extern YColor YGetColorCMYK(float c, float m, float y, float k);
extern YColor YColorSetAlpha(YColor color, uint8 alpha);

extern void YColorGetRGB(YColor color, uint8 *red, uint8 *green, uint8 *blue);
extern void YColorGetHSV(YColor color, float *h, float *s, float *v);
extern void YColorGetHLS(YColor color, float *h, float *l, float *s);
extern void YColorGetCMYK(YColor color, float *c, float *m, float *y,
						  float *k);
extern void YColorGetAlpha(YColor color, uint8 *alpha);

extern void YSetWindowBackground(YWindow *window, YColor background);
extern void YClearWindow(YWindow *window);

extern void YDrawLine(YGC *gc, YPoint *p1, YPoint *p2);
extern void YDrawPoints(YGC *gc, YPoint **points, int npoints);
extern void YDrawRectangle(YGC *gc, YRect *rect);
extern void YFillRectangle(YGC *gc, YRect *rect);

extern void YWinBlit(YGC *gc, YWindow *source,
					 YPoint *src_loc, YPoint *dest_loc, YRect *size);

extern YWindow *YRootWindowOfDisplay(YDisplay *display);

extern YWindow_id YGetWindowId(YWindow *window);
extern YWindow *YGetWindowById(YDisplay *display, YWindow_id id);
extern YImage *YGetImageById(YDisplay *display, YWindow_id id);

extern int YGetSubwindows(YWindow *window, YWindow ***sub_windows);
extern void YReleaseSubwindows(YWindow *window, YWindow **sub_windows, int length);

/* rectangle & point operations. */
extern YBool YDoRectsIntersect(YRect *r1, YRect *r2);
extern YBool YIsPointInRect(YPoint *p, YRect *r);
extern YRect *YRectIntersect(YRect *r1, YRect *r2);

/* font specific stuff */
extern int YListFonts(YDisplay *display, const char **font_specs);
extern YFont *YLoadFont(YDisplay *display, const char *font_spec);
extern void YDestroyFont(YDisplay *display, YFont *font);

/* text drawing and stuff */
extern void YDrawAsciiText(YGC *gc, YPoint *where, char *text);
extern void YDrawUTF8Text(YGC *gc, YPoint *where, char *text, int length);
extern int YTextWidth(YFont *font, char *string, int length);

/* file descriptor foo. */
typedef void (*YInputProc)(void *closure);

extern void YAddInput(int fd, YInputProc proc, void *closure);
extern void YRemoveInput(int fd, YInputProc proc, void *closure);

/* events */
void YAddInterest(YWindow *window);
void YNotifyInterest(YWindow *window, Y_Event *event);
void YGetNextEvent(YDisplay *display, Y_Event *event);

#endif /* _y_lib_h */
