/* -*- Mode: C; tab-width: 4 -*-
   ymessages.h -- message types for communicating between the client and
                  microserver.
   Created: Chris Toshok <toshok@hungry.com>, 9-May-1997
 */

#ifndef _y_messages_h
#define _y_messages_h

enum {
  /* requests */
  Y_GetRootWindowReq,
  Y_GetDriverReq,
  Y_LastRequest,	/* special, marks the end of the requests */

  /* replies */
  Y_GetRootWindowRep,
  Y_GetDriverRep,
  Y_LastReply, 		/* special, marks the end of the replies */

  /* events */
  Y_ExposeEvent,
  Y_KeyDownEvent,
  Y_KeyUpEvent,
  Y_MouseDownEvent,
  Y_MouseUpEvent,
  Y_MouseMotionEvent,
  Y_MouseEnterEvent,
  Y_MouseExitEvent
};

typedef struct {
  int32 tag;
  YWindow_id root_id;
} YGetRootWindowRep;

typedef struct {
  int32 tag;

  int32 first_id;
  int32 num_ids;
} YGetDriverRep;

typedef struct {
  int32 tag;
} YAnyRep;


typedef struct {
  int32 tag;
  int32 timestamp;
  YWindow_id target;
} YAnyEvent;

typedef struct {
  int32 tag;
  int32 timestamp;
  YWindow_id target;
  YRect bounds;
} YExposeEvent;

typedef struct {
  int32 tag;
  int32 timestamp;
  YWindow_id target;
  int32 key;
  int32 x;
  int32 y;
} YKeyDownEvent;

typedef struct {
  int32 tag;
  int32 timestamp;
  YWindow_id target;
  int32 key;
  int32 x;
  int32 y;
} YKeyUpEvent;

typedef struct {
  int32 tag;
  int32 timestamp;
  YWindow_id target;
  char button;
  int32 x;
  int32 y;
} YMouseDownEvent;

typedef struct {
  int32 tag;
  int32 timestamp;
  YWindow_id target;
  char button;
  int32 x;
  int32 y;
} YMouseUpEvent;

typedef struct {
  int32 tag;
  int32 timestamp;
  YWindow_id target;
  int32 x;
  int32 y;
} YMouseMotionEvent;

typedef struct {
  int32 tag;
  int32 timestamp;
  YWindow_id target;
} YMouseEnterEvent;

typedef struct {
  int32 tag;
  int32 timestamp;
  YWindow_id target;
} YMouseExitEvent;

typedef union
{
  /* all that's needed for requests, which are nothing but a tag. */
  int32 tag;
  
  /* replies */
  YAnyRep any_rep;
  YGetRootWindowRep getroot_rep;
  YGetDriverRep getdriver_rep;

  /* events */
  YAnyEvent any_event;
  YExposeEvent expose;
  YKeyDownEvent key_down;
  YKeyUpEvent key_up;
  YMouseDownEvent mouse_down;
  YMouseUpEvent mouse_up;
  YMouseMotionEvent mouse_motion;
  YMouseEnterEvent mouse_enter;
  YMouseExitEvent mouse_exit;
} Y_Message;

typedef Y_Message Y_Event;
typedef Y_Message Y_Reply;
typedef Y_Message Y_Request;

#endif /* _y_messages_h */
