/* -*- Mode: C; tab-width: 4 -*-
   ytypes.h -- basic, silly types for everything under the sun.
   Created: Chris Toshok <toshok@hungry.com>, 11-May-1997
 */

#ifndef _y_types_h
#define _y_types_h

#include <sys/types.h>

/* bring in the definitions of uint8/16/32 */
#if defined (__FreeBSD__) || defined(linux) || defined(sgi)
typedef u_int8_t uint8;
typedef u_int16_t uint16;
typedef u_int32_t uint32;
typedef int8_t int8;
typedef int16_t int16;
typedef int32_t int32;
typedef int64_t int64;
#endif

/* Irix (6.3) already has uint64 defined in <sys/types.h> */
#if defined(__FreeBSD__) || defined(linux)
typedef u_int64_t uint64;
#endif

#if defined(sun) || defined(hpux) || defined(__osf__)
typedef unsigned char uint8;
typedef unsigned short uint16;
typedef unsigned int uint32;
typedef unsigned long long uint64;

typedef char int8;
typedef short int16;
typedef int int32;
typedef long long int64;
#endif

/* define our boolean type */
typedef enum {Y_TRUE = 1, Y_FALSE = 0} YBool;

#ifndef NULL
#define NULL (void*)0
#endif

typedef struct {
  int x;
  int y;
} YPoint;

typedef struct {
  int left;
  int top;
  int right;
  int bottom;
} YRect;

/* our type for colors.  indices into the colormap. */
typedef uint32 YColor;

/* the id's for various types, for doing shm stuff, and for communicating
   to the network server. */
typedef uint32 YWindow_id;
typedef uint32 YGC_id;
typedef uint32 YFont_id;

typedef enum {Y_UNMAPPED = 0, Y_MAPPED = 1} YWindowMapState;

typedef struct YWindow_struct YWindow;
typedef struct YWindow_struct YImage;
typedef struct YGC_struct YGC;
typedef struct YGlyph_struct YGlyph;

typedef struct YSharedWindowInfo_struct {
  uint16 sem_id; /* our semaphore */

  uint32 ref_count; /* used for reference counting windows and images. */

  YWindow_id parent_id;

  YWindow_id prev_sibling;
  YWindow_id next_sibling;

  YWindow_id top_child;
  YWindow_id bottom_child;

  YRect bounds;
  YWindowMapState map_state;

  /* clients interested in events on this window */
  int32 interested_parties[100];

  int num_interested;

  YColor background;

} YSharedWindowInfo;

#endif /*_y_types_h*/
