/* -*- Mode: C; tab-width: 4 -*-
   Ydll.c -- Y interface to dynamic libraries.
   Created: Chris Toshok <toshok@hungry.com>, 30-Jun-1997
 */
/* 
   Copyright (C) 1996, 1997 The Hungry Programmers.
   
   This file is part of The Y Window System.
   
   Y is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
   
   Y is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.
   
   You should have received a copy of the GNU General Public License
   along with GLD; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307 , USA.
*/

#include "ydll.h"
#include "ydll_p.h"
#include "config.h"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#ifdef HAVE_DLFCN_H
#include <dlfcn.h>
#else
#ifdef HAVE_DL_H
#include <dl.h>
#else
#error port me
#endif
#endif

#if defined (__FreeBSD__)
#define NEED_UNDERSCORE
#elif defined (sgi) || defined(linux) || defined(sun /* XXX but not 4.1.3 */) || defined(hpux) || defined(__osf__)
/* do nothing. */
#else
#error port me!
#endif

YDLL *libs = NULL;

static YDLL *
ydll_findLibrary(const char *lib_path)
{
  YDLL *cur_lib;

  for (cur_lib = libs;
       cur_lib != NULL;
       cur_lib = cur_lib->next)
    {
      if (!strcmp(lib_path, cur_lib->name))
		{
		  return cur_lib;
		}
    }

  return NULL;
}

YDLL *
YDLL_LoadLibrary(const char *lib_path)
{
  YDLL *lib;
  if (!(lib = ydll_findLibrary(lib_path)))
    {
      lib = (YDLL*)calloc(1, sizeof(YDLL));
      
#ifdef HAVE_DLFCN_H
      lib->os_data = dlopen((char*)lib_path, RTLD_NOW);
#endif
#ifdef HAVE_DL_H
      lib->os_data = shl_load(lib_path, BIND_DEFERRED | BIND_NONFATAL, 0L);
#endif
      
      if (lib->os_data == NULL)
		{
#ifdef HAVE_DLFCN_H
		  fprintf(stderr, "YDLL_LoadLibrary: %s\n", dlerror());
#endif
		  free(lib);
		  return NULL;
		}
      
      lib->name = strdup(lib_path);

      lib->next = libs;
      libs = lib;
    }

  lib->ref_count++;
  return lib;
}

void
YDLL_UnloadLibrary(YDLL *dll)
{
  dll->ref_count --;
  if (dll->ref_count == 0)
    {
#ifdef HAVE_DLFCN_H
      dlclose(dll->os_data);
#endif
#ifdef HAVE_DL_H
      shl_unload(dll->os_data);
#endif
      if (dll->prev)
		dll->prev->next = dll->next;
      if (dll->next)
		dll->next->prev = dll->prev;

      if (libs == dll)
		libs = dll->next;

      free(dll->name);
      free(dll);
    }
}

void *
YDLL_LookupSymbol(YDLL *dll, const char *symbol_name)
{
  void *symbol;
  char *lookup;

  if (dll == NULL) return NULL;

#if defined (NEED_UNDERSCORE)
  lookup = (char*)malloc(strlen(symbol_name) + 2);
  strcpy (lookup, "_");
  strcat (lookup, symbol_name);
#else
  lookup = strdup(symbol_name);
#endif

#ifdef HAVE_DLFCN_H
  symbol = dlsym(dll->os_data, lookup);
#endif
#ifdef HAVE_DL_H
  /* returns an int value, 0 if successful, -1 if not */
  shl_findsym(dll->os_data, lookup, TYPE_UNDEFINED, symbol);
#endif
  free(lookup);

  return symbol;
}
