/* -*- Mode: C; tab-width: 4 -*-
   font.c -- fonts.
   Created: Chris Toshok <toshok@hungry.com>, 23-Sep-1997
*/
/* 
   Copyright (C) 1996, 1997 The Hungry Programmers.
   
   This file is part of The Y Window System.
   
   Y is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
   
   Y is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.
   
   You should have received a copy of the GNU General Public License
   along with GLD; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307 , USA.
*/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "ylib.h"
#include "yfonts_p.h"

/*
** Fonts are specified in the following format:
**
** font://renderer-name/font-name[?attribute=value[&attribute=value]*]
**
** or
**
** font:/font-name[?attribute=value[&attribute=value]*]
**
** The first choice allows the program or user to specify the actually
** font renderer to be used to render a given font.  The second choice
** should be used in all but the most extreme situations, as it gives
** the library the most flexibility -- it will find the renderer that
** is most appropriate for a given font at a given weight at a given
** point size.
**
** attributes are:
**
** weight:     value is a {bold,medium,light}  (e.g, weight=medium)
** point-size: value is a float (e.g., point-size=24.0)
** slant:      value is {regular,italic,oblique} (e.g, slant=italic)
**
** So, a complete example is:
**
** font:/helvetica?weight=medium&point-size=24.0
**
** and
**
** font://Hungry PostScript Font Renderer/helvetica?weight=medium&point-size=24.0&slant=oblique
*/

static YFont *
YFindFont(YDisplay *display,
		  const char *font_spec)
{
}

YFont *
YLoadFont(YDisplay *display, const char *font_spec)
{
  YFontRenderer *fonts_renderer;
  char *renderer_name;
  char *end_of_renderer_name = strstr (font_spec, ":");

  printf ("Loading font '%s'\n", font_spec);

  if (end_of_renderer_name == NULL)
	{
	  fprintf (stderr, "Malformed font spec: '%s'\n", font_spec);
	  return NULL;
	}

  renderer_name = (char *) calloc (end_of_renderer_name - font_spec + 1, 1);
  strncpy (renderer_name, font_spec, end_of_renderer_name - font_spec);

  printf (" Font renderer is '%s'\n", renderer_name);

  fonts_renderer = Y_GetFontRendererByName (renderer_name);
  if (fonts_renderer == NULL)
	{
	  fprintf (stderr, "YLoadFont couldn't get renderer for '%s'\n",
			   font_spec);
	  return NULL;
	}

  return (*fonts_renderer->_vtable.loadFont)(fonts_renderer, font_spec);
}


void
YDestroyFont(YDisplay *display, YFont *font)
{
  free(font->font_spec);

  YFontRenderer_destroy(font->renderer);

  free(font);
}


YFontRenderer *
YGetFontRendererForFont(YFont *font)
{
  return font->renderer;
}
