/* -*- Mode: C; tab-width: 4 -*-
   test-renderer.c -- test font renderer that doesn't do anything but draw a
                      rectangle where the letter should be.
   Created: Chris Toshok <toshok@hungry.com>, 10-Oct-1997
*/
/* 
   Copyright (C) 1996, 1997 The Hungry Programmers.
   
   This file is part of The Y Window System.
   
   Y is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
   
   Y is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.
   
   You should have received a copy of the GNU General Public License
   along with GLD; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307 , USA.
*/

#include "ylib.h"
#include "yfonts_p.h"

#include <string.h>
#include <stdlib.h>
#include <ctype.h>

static const char *
testGetName(YFontRenderer *renderer)
{
  return "Test Font Renderer";
}

static const char *
testGetVendor(YFontRenderer *renderer)
{
  return "Christoph Toshok";
}

static const char *
testGetVersion(YFontRenderer *renderer)
{
  return "$Revision: 1.2 $";
}

static YFont *
testLoadFont(YFontRenderer *renderer,
			 const char *font_spec)
{
  YFont *new_font = (YFont*)calloc(1, sizeof(YFont*));

  new_font->renderer = renderer;
  new_font->font_spec = strdup(font_spec);

  return new_font;
}

static int
testCountFonts(YFontRenderer *renderer)
{
  return 1;
}

static int
testListFonts(YFontRenderer *renderer,
			  const char ***font_specs)
{
  if (font_specs)
	{
	  *font_specs = (const char**)calloc(1, sizeof(char*));
	  (*font_specs)[0] = "font://Test Font Renderer/test-font";
	}

  return 1;
}

static void
testRenderCharacter(YFontRenderer *renderer,
					char *string, int length,
					YGC *gc,
					YPoint *start_at)
{
  YRect rect;

  if (!isspace(string[0]))
	{
	  rect.left = start_at->x;
	  rect.right = start_at->x + 12;
	  rect.bottom = start_at->y;
	  rect.top = start_at->y - 12;

	  YDrawRectangle(gc, &rect);
	}
}

static int
testMaxAscent(YFontRenderer *renderer,
			  YFont *font)
{
  return 12;
}

static int
testMaxDescent(YFontRenderer *renderer,
			   YFont *font)
{
  return 14;
}

static int
testMaxWidth(YFontRenderer *renderer,
			 YFont *font)
{
  return 14;
}

static int
testAdvancement(YFontRenderer *renderer,
				YFont *font,
				char character)
{
  return 14;
}

static int
testStringWidth(YFontRenderer *renderer,
				YFont *font,
				char *string,
				int string_length)
{
  return 14 * string_length;
}

static void
testDestroy(YFontRenderer *renderer)
{
}

YFontRenderer_vtable test_vtable = {
  testGetName,
  testGetVendor,
  testGetVersion,
  testLoadFont,
  testCountFonts,
  testListFonts,
  testRenderCharacter,
  testMaxAscent,
  testMaxDescent,
  testMaxWidth,
  testAdvancement,
  testStringWidth,
  testDestroy
};

static YFontRenderer test_renderer;

YFontRenderer *
YFontRenderer_get()
{
  test_renderer._vtable = test_vtable;

  return &test_renderer;
}
