/*--------------------------------------------------------------------------
  ----- File:        t1global.h
  ----- Author:      Rainer Menzner (rmz@neuroinformatik.ruhr-uni-bochum.de)
  ----- Date:        11/21/97
  ----- Description: This file is part of the t1-library. It contains
                     global declarations used by the t1-library.
  ----- Copyright:   This program is copyright (c) Rainer Menzner, 1996. 
                     Everyone is allowed to use the library and to make
		     changes to the source code, provided that the authors
		     name is left where it appears and that changes are
		     metioned in some notice. The library must be
		     distributed including sources, you are not allowed to
		     take money for this.
		     The parseAFM-package is copyrighted by Adobe Systems
		     Inc.
		     The type1 rasterizer is copyrighted by IBM and the
		     X11-consortium.
  ----- Warranties:  Of course, there's NO WARRANTY OF ANY KIND :-)
  ----- Credits:     I want to thank IBM and the X11-consortium for making
                     their rasterizer freely available.
		     Also thanks to Piet Tutelaers for his ps2pk, from
		     which I took the rasterizer sources in a format
		     independent from X11.
                     Thanks to all people who make free software living!
--------------------------------------------------------------------------*/
  

#include "t1misc.h"


/* Following struct will allow all accesses to font data! */
struct FONTBASE
{
  int no_fonts_ini;   /* The number of fonts initially declared in
			 Font database file. */
  int no_fonts;       /* The number of currently allocated fonts including
			 logical fonts produced by T1_CopyFont() */
  int no_fonts_limit; /* The maximum number of font for which is memory
			 currently available. */
  int bitmap_pad;     /* The value to which bitmap-scanlines are padded */
  int endian;         /* 1 if little endian representation and 0 otherwise */
  FONTPRIVATE *pFontArray; /* Points to first element of font pointer array */
  unsigned char **pFontFileNameIDArray; /* Points to the array, describing
					   the filenames */
} FontBase;
struct FONTBASE *pFontBase=NULL;



/* Further we need a struct where to save device dependent data such as
   resolution. The values in this struct may be altered using
   T1_SetDeviceRes(x_res,y_res). Generally, the fontsize is to be specified
   in BigPoints ("bp")---this is the default PostScript unit. It is
   1in=72bp.
   */
struct 
{
  float x_resolution;       /* Value must be specified in DPI */
  float y_resolution;       /* Value must be specified in DPI */
  float scale_x;            /* horizontal scale-value to get a
			       matrix scaled to 1b */
  float scale_y;             /* vertical scale-value to get a
			       matrix scaled to 1b */
} DeviceSpecifics;


struct stat filestats; /* A structure where fileinfo is stored */
unsigned char linebuf[BUF_SIZE];   /* A buffer for reading files line
					     by line */
int T1_Up=0;     /* This one is for initialization-checking */

/* The follwoing variable allows the primary rastering functions
   to check for the caller */
int SetFuncRequestID=0;

/* The following variable allows to suppress rastering at 1000 bp
   for getting a correct bounding box for slanted characters -->
   should only internally be used */
int ForceAFMBBox=0;

extern struct XYspace *IDENTITY; 

int errornumber;  /* for debugging purposes */
 
/* Endian=1 if BigEndian and 0 if Little Endian representation */
int Endian;

/* The following pointer should be set to path-strings--used for locating
   type1, afm and encoding files */
unsigned char pfab[]=".";
unsigned char afm[]=".";
unsigned char enc[]=".";
unsigned char fontdatabase[]="FontDataBase";
unsigned char *PFAB_ptr;
unsigned char *AFM_ptr;
unsigned char *ENC_ptr;
unsigned char *FDB_ptr;

/* We use a uchar buffer for error and warning messages: */
unsigned char err_warn_msg_buf[1024];

/* file pointer for log-file */
FILE *t1lib_log_file;
int  t1lib_log_level=2;

