/*--------------------------------------------------------------------------
  ----- File:        t1lib_x.h
  ----- Author:      Rainer Menzner (rmz@neuroinformatik.ruhr-uni-bochum.de)
  ----- Date:        11/29/97
  ----- Description: This file is part of the t1-library. It must be
                     included by the user of the t1lib. It contains
		     function declarations and some basic data types, the
		     user must deal with. This file is intended to be used
		     with X11
  ----- Copyright:   This program is copyright (c) Rainer Menzner, 1996. 
                     Everyone is allowed to use the library and to make
		     changes to the source code, provided that the authors
		     name is left where it appears and that changes are
		     metioned in some notice. The library must be
		     distributed including sources, you are not allowed to
		     take money for this.
		     The parseAFM-package is copyrighted by Adobe Systems
		     Inc.
		     The type1 rasterizer is copyrighted by IBM and the
		     X11-consortium.
  ----- Warranties:  Of course, there's NO WARRANTY OF ANY KIND :-)
  ----- Credits:     I want to thank IBM and the X11-consortium for making
                     their rasterizer freely available.
		     Also thanks to Piet Tutelaers for his ps2pk, from
		     which I took the rasterizer sources in a format
		     independent from X11.
                     Thanks to all people who make free software living!
--------------------------------------------------------------------------*/
  

#include "ylib.h"


#define T1LIB_Y_SUPPORT


/* type definitions, needed by the user: */

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif


/* the data structure which contains the character bitmap description */
typedef struct
{
  unsigned char *bits;  /* A pointer to the characters local bitmap */
  struct                /* A struct containing diverse metric information */
  {
    int ascent;
    int descent;
    int leftSideBearing;
    int rightSideBearing;
    int characterWidth;
  } metrics;
  void *pFontCacheInfo;
  unsigned long bpp;
} GLYPH;


#define KERNING     1
#define NO_KERNING  0
#define OPAQUE      1
#define TRANSPARENT 0

/* the bounding box data structure: */
typedef struct
{ 
   int llx;	/* lower left x-position  */
   int lly;	/* lower left y-position  */
   int urx;	/* upper right x-position */
   int ury;	/* upper right y-position */
} BBox;


/* A data that makes most important information available to user. */
typedef struct
{
  int      width;       /* The glyph's width */
  BBox     bbox;        /* The glyph's bounding box */
  int      numchars;    /* The number of characters in the glyph (string) */
  int      *charpos;    /* A pointer to an integer array were the horizontal
			   positions in (afm units) of the individual
			   characters in the string are stored */
} METRICSINFO;


#define  PAD(bits, pad)  (((bits)+(pad)-1)&-(pad))


/* def's for T1_InitLib() */
#define NO_LOGFILE 0
#define LOGFILE 1
/* logfile levels: */
#define T1LOG_ERROR     1
#define T1LOG_WARNING   2
#define T1LOG_STATISTIC 3
#define T1LOG_DEBUG     4


/* function declarations: */

/* from t1base.c */
extern void *T1_InitLib( int log);
extern int scanFontDBase( unsigned char *filename);
extern int T1_CloseLib( void);
extern void print_msg( unsigned char *func_ident, unsigned char *msg_txt);
extern void T1_PrintLog( unsigned char *func_ident, unsigned char *msg_txt, int level);
extern void T1_SetLogLevel( int level);
extern int CheckForInit(void);
extern int CheckForFontID( int FontID);
extern int test_for_t1_file( unsigned char *buffer );
extern unsigned char *T1_GetFontFileName( int FontID);
extern int  T1_Get_no_fonts(void);
extern int T1_SetDeviceResolutions( float x_res, float y_res);
extern int T1_CopyFont( int FontID);
extern int T1_QueryYSupport( void);
extern void bin_dump_c(unsigned char value);
extern void bin_dump_s(unsigned short value);
extern void bin_dump_l(unsigned long value);
extern int T1_SetBitmapPad( int pad);
extern int T1_GetBitmapPad( void);

/* from t1delete.c */
extern int T1_DeleteSize( int FontID, float size);
extern int T1_DeleteAllSizes( int FontID);
extern int T1_FreeGlyph( GLYPH *glyph);
extern int T1_DeleteFont( int FontID);

/* from t1enc.c */
extern unsigned char **ScanEncodingFile( char *FileName);
extern unsigned char **T1_LoadEncoding( unsigned char *FileName);
extern int T1_DeleteEncoding( unsigned char **encoding);
extern int T1_ReencodeFont( int FontID, unsigned char **Encoding);

/* from t1env.c */
extern int ScanConfigFile( unsigned char **pfabenv_ptr, 
			   unsigned char **afmenv_ptr,
			   unsigned char **encenv_ptr,
			   unsigned char **fontdatabase_ptr);
extern unsigned char *Env_GetCompletePath( unsigned char *FileName,
					   unsigned char *env_ptr );

/* from t1finfo.c */
extern int T1_GetKerning( int FontID, unsigned char char1,
			  unsigned char char2);
extern int T1_GetCharWidth( int FontID, unsigned char char1);
extern BBox T1_GetCharBBox( int FontID, unsigned char char1);
extern float T1_GetUnderlinePosition( int FontID);
extern float T1_GetUnderlineThickness( int FontID);
extern unsigned char *T1_GetPostScriptName( int FontID);
extern unsigned char *T1_GetCharName( int FontID, unsigned char char1);
extern int T1_QueryLigs( int FontID,
			 unsigned char char1,
			 unsigned char **successors,
			 unsigned char **ligatures);
extern int T1_GetEncodingIndex( int FontID, unsigned char *char1);
extern int T1_GetStringWidth( int FontID, unsigned char *string,
			      int len,  long spaceoff, int kerning);
extern BBox T1_GetStringBBox( int FontID, unsigned char *string,
			      int len,  long spaceoff, int kerning);
extern METRICSINFO T1_GetMetricsInfo( int FontID, unsigned char *string,
				      int len,  long spaceoff, int kerning);
extern BBox T1_GetFontBBox( int FontID);
extern unsigned char **T1_GetAllCharNames( int FontID);

/* from t1load.c */
extern int T1_LoadFont( int FontID);
extern int openFontMetricsFile( int FontID);
extern void *CreateNewFontSize( int FontID, float size, int aa);
extern void *GetLastFontSize( int FontID);
extern void *QueryFontSize( int FontID, float size, int aa);

/* from t1set.c */
extern GLYPH *T1_SetChar( int FontID, unsigned char charcode, 
			  float size, float angle);
extern GLYPH *T1_SetString( int FontID, unsigned char *string, int len,
			    long spaceoff, int kerning,
			    float size, float angle);
extern GLYPH *T1_CopyGlyph(GLYPH *glyph);

/* from t1trans.c */
extern int T1_ExtendFont( int FontID, double extend);
extern int T1_SlantFont( int FontID, double slant);

/* from t1aaset.c */
extern GLYPH *T1_AASetChar( int FontID, unsigned char charcode, 
			    float size, float angle);
extern GLYPH *T1_AASetString( int FontID, unsigned char *string, int len,
			      long spaceoff, int kerning,
			      float size, float angle);
extern int T1_AASetGrayValues(unsigned long white,
			      unsigned long gray75,
			      unsigned long gray50,
			      unsigned long gray25,
			      unsigned long black);
extern int T1_AASetBitsPerPixel( int bpp);


/* from t1ylib.c */
extern int T1_SetYParams( YDisplay *display );

extern GLYPH *T1_SetCharY( YGC *gc, int mode, int x, int y,
			   int FontID, unsigned char charcode,
			   float size, float angle);
extern GLYPH *T1_SetStringY( YGC *gc, int mode, int x, int y,
			     int FontID, unsigned char *string, int len,
			     long spaceoff, int kerning,
			     float size, float angle);
extern GLYPH *T1_AASetCharY( YGC *gc, int mode, int x, int y,
			     int FontID, unsigned char charcode,
			     float size, float angle);
extern GLYPH *T1_AASetStringY( YGC *gc, int mode, int x, int y,
			       int FontID, unsigned char *string, int len,
			       long spaceoff, int kerning,
			       float size, float angle);
extern int T1_ComputeAAColorsY( unsigned long fg,
				unsigned long bg,
				int nolevels);
void T1_LogicalPositionY( int pos_switch);

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif
