/*--------------------------------------------------------------------------
  ----- File:        t1load.c 
  ----- Author:      Rainer Menzner (rmz@neuroinformatik.ruhr-uni-bochum.de)
  ----- Date:        11/22/97
  ----- Description: This file is part of the t1-library. It contains
                     functions for loading fonts  and for managing size
		     dependent data.
  ----- Copyright:   This program is copyright (c) Rainer Menzner, 1996. 
                     Everyone is allowed to use the library and to make
		     changes to the source code, provided that the authors
		     name is left where it appears and that changes are
		     metioned in some notice. The library must be
		     distributed including sources, you are not allowed to
		     take money for this.
		     The parseAFM-package is copyrighted by Adobe Systems
		     Inc.
		     The type1 rasterizer is copyrighted by IBM and the
		     X11-consortium.
  ----- Warranties:  Of course, there's NO WARRANTY OF ANY KIND :-)
  ----- Credits:     I want to thank IBM and the X11-consortium for making
                     their rasterizer freely available.
		     Also thanks to Piet Tutelaers for his ps2pk, from
		     which I took the rasterizer sources in a format
		     independent from X11.
                     Thanks to all people who make free software living!
--------------------------------------------------------------------------*/
  
#define T1LOAD_C

/*#define REALLOC_VM */


#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>

#include "../type1/ffilest.h"
#include "../type1/types.h"
#include "parseAFM.h" 
#include "../type1/objects.h"
#include "../type1/spaces.h"
#include "../type1/util.h"
#include "../type1/fontfcn.h"
#include "../type1/blues.h"
#include "../type1/regions.h"


#include "t1types.h"
#include "t1extern.h"
#include "t1load.h"
#include "t1env.h"
#include "t1base.h"
#include "t1finfo.h"


extern unsigned char *vm_base; /* from fontfcn.c in initfont()! */
extern unsigned char *vm_used; /* from fontfcn.c in fontfcnA()! */

int fontfcnA( char *env, int *mode, psfont *Font_Ptr);



/* T1_LoadFont(FontID): Loads a Type1 font into memory and allocates all
   memory, necessary for this. */

int T1_LoadFont( int FontID)
{
  int i, j, k;
  unsigned char *FileName;
  int mode;  /* This is used by the type1-library for error reporting */   
  unsigned char *charname;
  
  /* The following vars are used for reallocation of VM */
  long tmp_size;
  char shift;
  char * tmp_ptr;

  extern void *StdEncArrayP;       /* For checking of a fonts encoding */
  extern void *ISOLatin1EncArrayP;

  /* These are for constructing the kerning lookup table: */
  PairKernData *pkd;
  KERN_ENTRY *kern_tbl;
  int char1, char2;
  
  
  if (CheckForInit())return(-1);

  i=CheckForFontID(FontID);
  if (i==1)
    return(0);      /* Font already loaded */
  if (i==-1)
    return(-1);     /* illegal FontID */

  /* Allocate memory for ps_font structure: */
  if ((pFontBase->pFontArray[FontID].pType1Data=(psfont *)malloc(sizeof(psfont)))==NULL){
    sprintf( err_warn_msg_buf,
	     "Failed to allocate memory for psfont-struct (FontID=%d)", FontID);
    print_msg("T1_LoadFont()", err_warn_msg_buf);
    return(-1);
  }
  
  /* Fetch the full path of type1 font file */
  FileName=Env_GetCompletePath( T1_GetFontFileName(FontID),
				PFAB_ptr);
  
  /* And load all PostScript information into memory */
  if (fontfcnA( FileName, &mode,
		pFontBase->pFontArray[FontID].pType1Data) == FALSE){
    sprintf( err_warn_msg_buf,
	     "Loading font with ID = %d failed! (mode = %d)",
	    FontID, mode);
    print_msg( "T1_LoadFont()", err_warn_msg_buf);
    free(FileName);
    pFontBase->pFontArray[FontID].pType1Data=NULL;
    return(-1);
  }
  free(FileName);

  
  /* Store the base address of virtual memory and realloc in order not
     to waste too much memory: */
  pFontBase->pFontArray[FontID].vm_base=vm_base; 
  /* Get size of VM, ... */
  tmp_size=((long)vm_used - (long)vm_base); 
  /* ... realloc to that size ... */
#ifdef REALLOC_VM
  tmp_ptr=(char *)realloc(vm_base,  tmp_size); 
  /* ... and shift all pointers refering to that area */
  
  shift= (long)tmp_ptr - (long)vm_base;

  /* We start by shifting the topmost pointers: */
  pFontBase->pFontArray[FontID].vm_base=tmp_ptr;
  (unsigned long)(pFontBase->pFontArray[FontID].pType1Data->vm_start) +=shift;
  (unsigned long)pFontBase->pFontArray[FontID].pType1Data->CharStringsP +=shift;
  (unsigned long)pFontBase->pFontArray[FontID].pType1Data->Private +=shift;
  (unsigned long)pFontBase->pFontArray[FontID].pType1Data->fontInfoP +=shift;
  (unsigned long)(pFontBase->pFontArray[FontID].pType1Data->BluesP) +=shift;
  
  /* We now have to care for correcting all pointers which are in the VM
     and refer to some place in the VM! Note: Instead of selecting the
     appropriate pointer-elements of the union we simply shift the unspecified
     pointer "valueP".
     Note: The filename entry does not need to be modified since it does not
     need to be shifted since it points to memory managed by t1lib.
     */
  /* FontInfo-dictionary: All name-pointers and the pointers to all array
     types have to be shifted: */
  i=pFontBase->pFontArray[FontID].pType1Data->fontInfoP[0].key.len;
  for (j=1; j<=i; j++){
    if ((pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.type==OBJ_ARRAY) ||
	(pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.type==OBJ_STRING) ||
	(pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.type==OBJ_NAME) ||
	(pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.type==OBJ_FILE)){
      (long)pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.valueP +=shift;
    }
    /* The encoding needs special treatment: */
    if (pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.type==OBJ_ENCODING){
      /* If a builtin encoding is used, it is sufficient to shift the pointer
	 to the Encoding since the character-namestrings of builtin encodings
	 are static and thus located on the heap.
	 For font-specific encoding, character-namestrings reside in VM and thus
	 each entry has to be shifted. */
      if ((pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.arrayP
	   == StdEncArrayP) ||
	  (pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.arrayP
	   == ISOLatin1EncArrayP)){ /* Font uses builtin encoding */
	(long)pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.valueP +=shift;
	;
      } 
      else{ /* Font-specific encoding */ 
	(long)pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.valueP +=shift;
	for (k=0; k<256; k++){
	  (long) pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.arrayP[k].data.arrayP += shift;
	}
      }
    } /* end of encoding-handling */
    (long)pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].key.data.valueP +=shift;
  } /* fontinfo-dict done */

  /* Private-dictionary: All name-pointers and the pointers to all array
     types have to be shifted: */
  i=pFontBase->pFontArray[FontID].pType1Data->Private[0].key.len;
  for (j=1; j<=i; j++){
    if ((pFontBase->pFontArray[FontID].pType1Data->Private[j].value.type==OBJ_ARRAY) ||
	(pFontBase->pFontArray[FontID].pType1Data->Private[j].value.type==OBJ_STRING) ||
	(pFontBase->pFontArray[FontID].pType1Data->Private[j].value.type==OBJ_NAME) ||
	(pFontBase->pFontArray[FontID].pType1Data->Private[j].value.type==OBJ_FILE)){
      (long)pFontBase->pFontArray[FontID].pType1Data->Private[j].value.data.valueP +=shift;
    }
    (long)pFontBase->pFontArray[FontID].pType1Data->Private[j].key.data.valueP +=shift;
  }
  
  /* BluesP: The entry "next" is the only pointer in blues_struct. Although it is
     not used anywhere we should shift it for correctness reasons (in case its not
     NULL)! */
  if (pFontBase->pFontArray[FontID].pType1Data->BluesP->next != NULL)
    (long)pFontBase->pFontArray[FontID].pType1Data->BluesP->next +=shift;

  /* The CharStrings-dictionary: Every namepointer and its corresponding
     charstring has to be shifted: */
  i=pFontBase->pFontArray[FontID].pType1Data->CharStringsP[0].key.len;
  for (j=1; j<=i; j++){
    (long)pFontBase->pFontArray[FontID].pType1Data->CharStringsP[j].value.data.valueP +=shift;
    (long)pFontBase->pFontArray[FontID].pType1Data->CharStringsP[j].key.data.valueP +=shift;
  }

  /* The Subroutines have also to be reorganized: */
  i=pFontBase->pFontArray[FontID].pType1Data->Subrs.len;
  /* First, shift pointer to array-start and after that the pointers to
     each command string: */
  (long)pFontBase->pFontArray[FontID].pType1Data->Subrs.data.arrayP +=shift;
  for (j=1; j<=i; j++) {
    (long)pFontBase->pFontArray[FontID].pType1Data->Subrs.data.arrayP[j].data.valueP +=shift;
  }
#endif
  

  /* Generate a message how much VM the current font consumes */
  sprintf( err_warn_msg_buf,
	   "VM for Font %d: %d bytes", FontID, (int) tmp_size);
  T1_PrintLog( "T1_LoadFont()", err_warn_msg_buf, T1LOG_STATISTIC);

  
  /* Set the matrix for common transformations to "no transformations" */
  pFontBase->pFontArray[FontID].FontTransform[0]=1.0;
  pFontBase->pFontArray[FontID].FontTransform[1]=0.0;
  pFontBase->pFontArray[FontID].FontTransform[2]=0.0;
  pFontBase->pFontArray[FontID].FontTransform[3]=1.0;

  /* Now, that the font has been loaded into memory, try to find the
     FontMatrix in the font info dictionary. If it exists, load it into
     our local fontmatrix, otherwise use a default matrix which scales to
     1/1000 (since font outlines  are defined in a 1000 point space)
     and does no further transformations. */
  if (pFontBase->pFontArray[FontID].pType1Data->fontInfoP[FONTMATRIX].value.data.arrayP == NULL){
    pFontBase->pFontArray[FontID].FontMatrix[0]=0.001;
    pFontBase->pFontArray[FontID].FontMatrix[1]=0.0;
    pFontBase->pFontArray[FontID].FontMatrix[2]=0.0;
    pFontBase->pFontArray[FontID].FontMatrix[3]=0.001;
  }
  else{
    pFontBase->pFontArray[FontID].FontMatrix[0]= pFontBase->pFontArray[FontID].pType1Data->fontInfoP[FONTMATRIX].value.data.arrayP[0].data.real;
    pFontBase->pFontArray[FontID].FontMatrix[1]= pFontBase->pFontArray[FontID].pType1Data->fontInfoP[FONTMATRIX].value.data.arrayP[1].data.real;
    pFontBase->pFontArray[FontID].FontMatrix[2]= pFontBase->pFontArray[FontID].pType1Data->fontInfoP[FONTMATRIX].value.data.arrayP[2].data.real;
    pFontBase->pFontArray[FontID].FontMatrix[3]= pFontBase->pFontArray[FontID].pType1Data->fontInfoP[FONTMATRIX].value.data.arrayP[3].data.real;
  }

  /* Set the default values for transformation: */
  pFontBase->pFontArray[FontID].slant=0.0;
  pFontBase->pFontArray[FontID].extend=1.0;
  

  /* Now try to load afm-structures from corresponding .afm-file. */
  if ((i=openFontMetricsFile( FontID))){
    /* Try a fallback function: */
    if ((i=openFontMetricsFileSloppy( FontID))){
      sprintf( err_warn_msg_buf,
	       "Warning: Error (%d) processing afm-file for Font %d!",
	       i ,FontID);
      T1_PrintLog( "T1_LoadFont()", err_warn_msg_buf, T1LOG_WARNING);
      T1_PrintLog( "T1_LoadFont()", "Leaving afm-structure in unknown state", T1LOG_WARNING);
      T1_PrintLog( "T1_LoadFont()", "Segmentation fault might result",T1LOG_WARNING);
    }
    else{
      sprintf( err_warn_msg_buf,
	       "Warning: Limited afm-information for Font %d",FontID);
      T1_PrintLog( "T1_LoadFont()", err_warn_msg_buf, T1LOG_WARNING);
      T1_PrintLog( "T1_LoadFont()", "Using only character metrics data", T1LOG_WARNING);
    }
  }

  /* Now, set Encodingvector entry to NULL, thus indicating that by default the
     fonts internal encoding should be used
     */
  pFontBase->pFontArray[FontID].pFontEnc=NULL;

  
  /* If AFM-Info available we try to speed up some things: */
  if (pFontBase->pFontArray[FontID].pAFMData != NULL){
    /* We have to fill the array that maps the current encodings' indices to the
       indices used in afm file */
    if ((pFontBase->pFontArray[FontID].pAFMData->enc_map=
	 (int *)calloc(256,sizeof(int)))==NULL){
      return(-1);
    }
    for (i=0; i<256; i++)
      pFontBase->pFontArray[FontID].pAFMData->enc_map[i]=-1;
    for (i=0; i<256; i++){
      charname=T1_GetCharName( FontID, i);
      for ( j=0; j<pFontBase->pFontArray[FontID].pAFMData->numOfChars; j++){
	if (strcmp( charname,
		    pFontBase->pFontArray[FontID].pAFMData->cmi[j].name)==0){
	  pFontBase->pFontArray[FontID].pAFMData->enc_map[i]=j;
	}
      }
    }
    
    /* We now introduce a "flag map" which gives quickly tells what metric
       features a character provides. */
    if ((pFontBase->pFontArray[FontID].pAFMData->flags= 
	 (unsigned char *)calloc(256,sizeof(unsigned char)))==NULL){ 
      return(-1); 
    } 
    
    /* We now create an encoding-specific kerning table which will speed up
       looking for kerning pairs! */
    /* First, get number of defined kerning pairs: */
    k=pFontBase->pFontArray[FontID].pAFMData->numOfPairs;
    if ((pFontBase->pFontArray[FontID].pAFMData->kern_tbl=
	 (KERN_ENTRY *)malloc( k*sizeof( KERN_ENTRY)))==NULL){
      return(-1);
    }
    kern_tbl=pFontBase->pFontArray[FontID].pAFMData->kern_tbl;
    pkd=pFontBase->pFontArray[FontID].pAFMData->pkd;
    j=0;
    for ( i=0; i<k; i++){
      if ((char1=T1_GetEncodingIndex( FontID, pkd[i].name1))==-1)
	/* pair is not relevant in current encoding */
	continue;
      if ((char2=T1_GetEncodingIndex( FontID, pkd[i].name2))==-1)
	/* pair is not relevant in current encoding */
	continue;
      /* Since we get here we have a relevant pair -->
	 Put char1 in higher byte and char2 in LSB:
	 */
      kern_tbl[j].chars=(char1 << 8) | char2;
      /* We only make use of horizontal kerning */
      kern_tbl[j].hkern=pkd[i].xamt;
      j++;
      /* Set the kerning flag for this char1: */
      pFontBase->pFontArray[FontID].pAFMData->flags[char1] |=KERN_FLAG;
    }
    /* Reset the remaining of the table in case there were
       irrelevant pairs: */
    for ( ; j<k; j++){
      kern_tbl[j].chars=0;
      kern_tbl[j].hkern=0;
    }
    
  } /* End of "if (AFM-info ..)" */
  
  
  /* We have just loaded a physical font into memory, thus .... */
  pFontBase->pFontArray[FontID].physical=1;

  /* Set reference-counter to 1: */
  pFontBase->pFontArray[FontID].refcount=1;

  /* Get the index into encoding vector where the space character is
     found. If not encoded, set space_position to -1. */
  pFontBase->pFontArray[FontID].space_position=-1;
  i=0;
  while (i<256){
    if (strcmp( (unsigned char *)pFontBase->pFontArray[FontID].pType1Data->fontInfoP[ENCODING].value.data.arrayP[i].data.arrayP,
		"space")==0){
      /* space found at position i: */
      pFontBase->pFontArray[FontID].space_position=i;
      break;
    }
    i++;
  }
  
  /* Finally, set the font size dependencies pointer to NULL since we can
     assume, that at load time of a font, no size specific data of this
     font is available.
     */
  
  pFontBase->pFontArray[FontID].pFontSizeDeps=NULL;

  /* If wanted, some debugging information is put into logfile */
  sprintf( err_warn_msg_buf, "Pointer vm_base: 0x%X",
	   (int)pFontBase->pFontArray[FontID].vm_base);
  T1_PrintLog( "T1_LoadFont()", err_warn_msg_buf, T1LOG_DEBUG);
  sprintf( err_warn_msg_buf, "Pointer vm_start: 0x%X",
	   (int)pFontBase->pFontArray[FontID].pType1Data->vm_start);
  T1_PrintLog( "T1_LoadFont()", err_warn_msg_buf, T1LOG_DEBUG);
  sprintf( err_warn_msg_buf, "Pointer CharStringsP: 0x%X",
	   (int)pFontBase->pFontArray[FontID].pType1Data->CharStringsP);
  T1_PrintLog( "T1_LoadFont()", err_warn_msg_buf, T1LOG_DEBUG);
  sprintf( err_warn_msg_buf, "Pointer Private: 0x%X",
	   (int)pFontBase->pFontArray[FontID].pType1Data->Private);
  T1_PrintLog( "T1_LoadFont()", err_warn_msg_buf, T1LOG_DEBUG);
  sprintf( err_warn_msg_buf, "Pointer fontInfoP: 0x%X",
	   (int)pFontBase->pFontArray[FontID].pType1Data->fontInfoP);
  T1_PrintLog( "T1_LoadFont()", err_warn_msg_buf, T1LOG_DEBUG);

  return(0);
}
  

/* openFontMetricsFile( FontID): Gets the fontfilename corresponding to
   FontID, opens the corresponding afm-file and fills the data structures.
   return-value is the value returned by the parseFile() function: */

int openFontMetricsFile( int FontID)
{
  unsigned char *FileName;
  int i;
  FILE *metricsfile;
  extern int errno;
  

  FileName=T1_GetFontFileName( FontID);
  i=strlen(FileName);
  /* We now change the extension pfa or pfb to afm: */
  FileName[i-3]='a';
  FileName[i-2]='f';
  FileName[i-1]='m';

  /* Get full path of the afm file */
  FileName=Env_GetCompletePath( FileName, AFM_ptr);
  
  /* open afm-file: */
  if ((metricsfile=fopen(FileName,"r"))==NULL){
    if (FileName!=NULL)
      free(FileName);
    return(-4);
  }
  free(FileName);
  
  /* Call procedure to read afm-file and store the data formatted.
     Flags used here: P_M  All Metrics Information
                      P_P  Pair Kerning Information
     The P_G flag to get global font information should not be used
     if not absolutely needed. When parsing an unknown keyword, which
     may be harmless, the parseFile function returns the error code
     -1 (parseError). On the other hand, all other really relevant
     data may habe been parsed and stored correctly. In such a case,
     There's no way to make a serious decision whether an error has
     occured or not.
     */
  i=parseFile( (FILE *) metricsfile,
	       (FontInfo **) &(FontBase.pFontArray[FontID].pAFMData),
	       P_M | P_P );
  fclose(metricsfile);
  return(i);
}



/* openFontMetricsFileSloppy( FontID): Gets the fontfilename corresponding to
   FontID, opens the corresponding afm-file and tries to fill the data
   structures only with metric data. This can be considered a fallback!
   */
int openFontMetricsFileSloppy( int FontID)
{
  unsigned char *FileName;
  int i;
  FILE *metricsfile;
  

  FileName=T1_GetFontFileName( FontID);
  i=strlen(FileName);
  /* We now change the extension pfa or pfb to afm: */
  FileName[i-3]='a';
  FileName[i-2]='f';
  FileName[i-1]='m';

  /* Get full path of the afm file */
  FileName=Env_GetCompletePath( FileName, AFM_ptr);
  /* open afm-file: */
  if ((metricsfile=fopen(FileName,"r"))==NULL){
    free(FileName);
    return(-4);
  }
  free(FileName);
  
  /* Call procedure to read afm-file and store the data formatted.
     Flags used here: P_M  All Metrics Information
     */
  i=parseFile( (FILE *) metricsfile,
	       (FontInfo **) &(FontBase.pFontArray[FontID].pAFMData),
	       P_M );
  fclose(metricsfile);
  return(i);
}




/* CreateNewFontSize( FontID, size): Create a new size "size" of font
   "FontID" and allocate all data necessary for this. The data
   structure is connected to the linked list of FontSizeDeps for this
   font. Returns a pointer to the newly created FontSizeDeps-struct
   if all went correct and NULL otherwise.
   Since of version 0.3 a member antialias has been added to the
   FONTSIZEDEPS structure!
   */
FONTSIZEDEPS *CreateNewFontSize( int FontID, float size, int aa)
{

  FONTSIZEDEPS *pFontSizeDeps, *pPrev;
  

  /* First, get to the last font size in the linked list for this font.
     The following routine returns the address of the last struct in the
     linked list of FONTSIZEDEPS or NULL if none exists. */
  pFontSizeDeps=GetLastFontSize( FontID);
  pPrev=pFontSizeDeps;
  
  
  if (pFontSizeDeps==NULL){
    /* Allocate memory for first FontSizeDeps-structure: */
    if ((pFontBase->pFontArray[FontID].pFontSizeDeps=(FONTSIZEDEPS *)malloc(sizeof(FONTSIZEDEPS)))==NULL)
      return(NULL);
    pFontSizeDeps=pFontBase->pFontArray[FontID].pFontSizeDeps;
  }
  else{
    /* A valid address of an existing structure was found */
    if ((pFontSizeDeps->pNextFontSizeDeps=(FONTSIZEDEPS *)malloc(sizeof(FONTSIZEDEPS)))==NULL)
      return(NULL);
    pFontSizeDeps=pFontSizeDeps->pNextFontSizeDeps;
  }

  /* The pointer to the previous struct */
  pFontSizeDeps->pPrevFontSizeDeps=pPrev;
  /* Put the size into this structure */
  pFontSizeDeps->size=size;
  /* Set the antialias mark: */
  if (aa)
    pFontSizeDeps->antialias=1;
  else
    pFontSizeDeps->antialias=0;
  
  /* Just the current becomes now the last item in the linked list: */
  pFontSizeDeps->pNextFontSizeDeps=NULL;
  /* Setup CharSpaceMatrix for this font: */
  pFontSizeDeps->pCharSpaceLocal=(struct XYspace *)Permanent(CopySpace(IDENTITY));
  /* Apply transformation with font matrix: */
  pFontSizeDeps->pCharSpaceLocal=(struct XYspace *)Transform(pFontSizeDeps->pCharSpaceLocal,
							     pFontBase->pFontArray[FontID].FontMatrix[0],
							     pFontBase->pFontArray[FontID].FontMatrix[1],
							     pFontBase->pFontArray[FontID].FontMatrix[2],
							     pFontBase->pFontArray[FontID].FontMatrix[3]);
  /* Apply a further transformation (optionally): */
  pFontSizeDeps->pCharSpaceLocal=(struct XYspace *)Transform(pFontSizeDeps->pCharSpaceLocal,
							     pFontBase->pFontArray[FontID].FontTransform[0],
							     pFontBase->pFontArray[FontID].FontTransform[1],
							     pFontBase->pFontArray[FontID].FontTransform[2],
							     pFontBase->pFontArray[FontID].FontTransform[3]);
  /* Apply device specific scaling */
  pFontSizeDeps->pCharSpaceLocal=(struct XYspace *)Scale(pFontSizeDeps->pCharSpaceLocal,
							 DeviceSpecifics.scale_x,
							  - DeviceSpecifics.scale_y);
  /* Apply desired scaling factor */
  pFontSizeDeps->pCharSpaceLocal=(struct XYspace *)Scale(pFontSizeDeps->pCharSpaceLocal,
							 size, size);
							 
  /* We should now allocate memory for the glyph area of the font
     cache: */
  if ((pFontSizeDeps->pFontCache=(GLYPH *)calloc(256,sizeof(GLYPH)))
      ==NULL)
    return(NULL);

  sprintf( err_warn_msg_buf, "New Size %f created for FontID %d (antialias=%d)",
	   pFontSizeDeps->size, FontID, pFontSizeDeps->antialias);
  T1_PrintLog( "CreateNewFontSize()", err_warn_msg_buf, T1LOG_STATISTIC);
  /* We are done */
  return(pFontSizeDeps);
  
}

  


/* QueryFontSize( FontID, size, aa): Search if a requested size of font
   FontID is already existing. If so, it returns a pointer to the
   respective FontSizeDeps-structure,  otherwise NULL is returned: */
FONTSIZEDEPS *QueryFontSize( int FontID, float size, int aa)
{
  
  FONTSIZEDEPS *link_ptr;


  /* There's not yet one size: */
  if (pFontBase->pFontArray[FontID].pFontSizeDeps == NULL)
    return(pFontBase->pFontArray[FontID].pFontSizeDeps);
  

  /* There's already existing one or more size */
  link_ptr=pFontBase->pFontArray[FontID].pFontSizeDeps;
  
  while (((link_ptr->size != size)||(link_ptr->antialias != aa))
	 &&(link_ptr->pNextFontSizeDeps != NULL))
    link_ptr=link_ptr->pNextFontSizeDeps;
  
  if ((link_ptr->size != size)||(link_ptr->antialias != aa))
    return( NULL);     /* requested size/aa-combination  was not found */
  else
    return(link_ptr); /* return pointer to requested struct */
  
}

/* FONTSIZEDEPS *GetLastFontSize( FontID): Get the address of the
   last struct in the linked list of FontSizeDeps or NULL if there is
   no existing size dependent data. */
FONTSIZEDEPS *GetLastFontSize( int FontID)
{
  FONTSIZEDEPS *link_ptr, *result_ptr;
  

  /* There's not yet one size: */
  if (pFontBase->pFontArray[FontID].pFontSizeDeps == NULL)
    return((FONTSIZEDEPS *) (pFontBase->pFontArray[FontID].pFontSizeDeps));
  
  
  /* There's already existing one or more size */
  link_ptr=pFontBase->pFontArray[FontID].pFontSizeDeps;
  
  while (link_ptr != NULL){
    result_ptr=link_ptr;
    link_ptr=link_ptr->pNextFontSizeDeps;
  }

  return((FONTSIZEDEPS *)(result_ptr));
}


