/* -*- Mode: C; tab-width: 4 -*-
   test-renderer.c -- test font renderer that doesn't do anything but draw a
                      rectangle where the letter should be.
   Created: Chris Toshok <toshok@hungry.com>, 10-Oct-1997
*/
/* 
   Copyright (C) 1996, 1997 The Hungry Programmers.
   
   This file is part of The Y Window System.
   
   Y is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
   
   Y is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.
   
   You should have received a copy of the GNU General Public License
   along with GLD; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307 , USA.
*/

#include "ylib.h"
#include "ylib_p.h"
#include "yfonts_p.h"

#include "t1lib.h"

#include <string.h>
#include <stdlib.h>
#include <ctype.h>

static const char *
t1GetName(YFontRenderer *renderer)
{
  return "T1 Font Renderer";
}

static const char *
t1GetVendor(YFontRenderer *renderer)
{
  return "Christoph Toshok";
}

static const char *
t1GetVersion(YFontRenderer *renderer)
{
  return "$Revision: 1.2 $";
}

static YFont *
t1LoadFont(YFontRenderer *renderer,
		   const char *font_spec)
{
  YFont *new_font = (YFont*)calloc(1, sizeof(YFont*));
  int status;
  static int inited = 0;
  int i;

  if (!inited)
	{
	  if (!T1_InitLib(T1LOG_DEBUG))
		{
		  printf ("Error initializating the type1 font renderer.\n");
		  free(new_font);
		  return NULL;
		}

	  T1_LoadEncoding("IsoLatin1.enc");
	  T1_LogicalPositionY(0);
	  printf ("Type1 Library initialized.  There are %d fonts available.\n", T1_Get_no_fonts());
	  inited = 1;
	}

  new_font->renderer = renderer;
  new_font->font_spec = strdup(font_spec);

  for (i = 0; i < T1_Get_no_fonts() - 1; i ++)
	if (strstr(font_spec, T1_GetFontFileName(i)))
	  {
		status = T1_LoadFont(i);
		break;
	  }

  new_font->font_data = (void*)i;

  printf ("Loaded font '%s'(%s)\n", T1_GetPostScriptName(i), T1_GetFontFileName(i));

  return new_font;
}

static int
t1CountFonts(YFontRenderer *renderer)
{
  return 1;
}

static int
t1ListFonts(YFontRenderer *renderer,
			  const char ***font_specs)
{
  if (font_specs)
	{
	  *font_specs = (const char**)calloc(1, sizeof(char*));
	  (*font_specs)[0] = "font://T1 Font Renderer/t1-font";
	}

  return 1;
}

static void
t1RenderCharacter(YFontRenderer *renderer,
				  char *string, int length,
				  YGC *gc,
				  YPoint *start_at)
{
  int font_id = (int)gc->font->font_data;
  GLYPH *g;

  T1_SetYParams(gc->window->display);
  T1_AASetGrayValues(YGetColorRGB(255,255,255),
					 YGetColorRGB(192,192,192),
					 YGetColorRGB(128,128,128),
					 YGetColorRGB(64,64,64),
					 YGetColorRGB(0,0,0));
  g = T1_SetString(font_id, string, length, 0, 1, 32.0, 0.0);

  T1_SetStringY(gc, 0, start_at->x, start_at->y - g->metrics.ascent,
				font_id, string, length, 0, 1, 32.0, 0.0);
}

static int
t1MaxAscent(YFontRenderer *renderer,
			YFont *font)
{
  return 12;
}

static int
t1MaxDescent(YFontRenderer *renderer,
			 YFont *font)
{
  return 14;
}

static int
t1MaxWidth(YFontRenderer *renderer,
		   YFont *font)
{
  return 14;
}

static int
t1Advancement(YFontRenderer *renderer,
			  YFont *font,
			  char character)
{
  GLYPH *glyph;
  int font_id = (int)font->font_data;

  glyph = T1_SetChar(font_id, character, 32.0, 0.0);

  return glyph->metrics.characterWidth;
}

static int
t1StringWidth(YFontRenderer *renderer,
			  YFont *font,
			  char *string,
			  int string_length)
{
  METRICSINFO info;
  int font_id = (int)font->font_data;

  info = T1_GetMetricsInfo(font_id, string, string_length, 0 ,0);

  return info.width;
}

static void
t1Destroy(YFontRenderer *renderer)
{
}

YFontRenderer_vtable t1_vtable = {
  t1GetName,
  t1GetVendor,
  t1GetVersion,
  t1LoadFont,
  t1CountFonts,
  t1ListFonts,
  t1RenderCharacter,
  t1MaxAscent,
  t1MaxDescent,
  t1MaxWidth,
  t1Advancement,
  t1StringWidth,
  t1Destroy
};

static YFontRenderer t1_renderer;

YFontRenderer *
YFontRenderer_get()
{
  t1_renderer._vtable = t1_vtable;

  return &t1_renderer;
}
