/*--------------------------------------------------------------------------
  ----- File:        t1trans.c 
  ----- Author:      Rainer Menzner (rmz@neuroinformatik.ruhr-uni-bochum.de)
  ----- Date:        5/18/97
  ----- Description: This file is part of the t1-library. It contains
                     functions for transforming fonts.
  ----- Copyright:   This program is copyright (c) Rainer Menzner, 1996. 
                     Everyone is allowed to use the library and to make
		     changes to the source code, provided that the authors
		     name is left where it appears and that changes are
		     metioned in some notice. The library must be
		     distributed including sources, you are not allowed to
		     take money for this.
		     The parseAFM-package is copyrighted by Adobe Systems
		     Inc.
		     The type1 rasterizer is copyrighted by IBM and the
		     X11-consortium.
  ----- Warranties:  Of course, there's NO WARRANTY OF ANY KIND :-)
  ----- Credits:     I want to thank IBM and the X11-consortium for making
                     their rasterizer freely available.
		     Also thanks to Piet Tutelaers for his ps2pk, from
		     which I took the rasterizer sources in a format
		     independent from X11.
                     Thanks to all people who make free software living!
--------------------------------------------------------------------------*/
  
#define T1TRANS_C


#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>

#include "../type1/ffilest.h"
#include "../type1/types.h"
#include "parseAFM.h" 
#include "../type1/objects.h"
#include "../type1/spaces.h"
#include "../type1/util.h"
#include "../type1/fontfcn.h"
#include "../type1/regions.h"


#include "t1types.h"
#include "t1extern.h"
#include "t1trans.h"
#include "t1base.h"


/* T1_ExtendFont(): Extend the font referenced by FontID by the factor
   extend. This is only allowed if no size dependent data exists.
   Of course, the font must already have been loaded.
   Returns 0 for success and -1 otherwise.
   */
int T1_ExtendFont( int FontID, double extend)
{

  /* First, check for font residing in memory: */
  if (CheckForFontID(FontID)!=1)
    return(-1);
  
  /* Second, check whether size-dependent data exists: */
  if (pFontBase->pFontArray[FontID].pFontSizeDeps != NULL)
    return(-1); 
  pFontBase->pFontArray[FontID].extend=extend;
  /*
  pFontBase->pFontArray[FontID].FontTransform[0] = pFontBase->pFontArray[FontID].FontTransform[0] * extend;
  */
  pFontBase->pFontArray[FontID].FontTransform[0] = extend;
  return(0);
}



/* T1_SlantFont(): Slant the font referenced by FontID by the factor
   extend. This is only allowed if no size dependent data exists.
   Of course, the font must already have been loaded.
   Returns 0 for success and -1 otherwise.
   */
int T1_SlantFont( int FontID, double slant)
{
  
  /* First, check for font residing in memory: */
  if (CheckForFontID(FontID)!=1)
    return(-1);
  
  /* Second, check whether size-dependent data exists: */
  if (pFontBase->pFontArray[FontID].pFontSizeDeps != NULL)
    return(-1); 
  pFontBase->pFontArray[FontID].slant=slant;
  /*
  pFontBase->pFontArray[FontID].FontTransform[2] = pFontBase->pFontArray[FontID].FontTransform[0] * slant;
  */
  pFontBase->pFontArray[FontID].FontTransform[2] = slant;
  return(0);
}



