/*--------------------------------------------------------------------------
  ----- File:        t1ylib.c 
  ----- Author:      Rainer Menzner (rmz@neuroinformatik.ruhr-uni-bochum.de)
  ----- Date:        12/02/97
  ----- Description: This file is part of the t1-library. It contains
                     functions for generating glyphs with data in
		     X11-Pixmap format.
  ----- Copyright:   This program is copyright (c) Rainer Menzner, 1996. 
                     Everyone is allowed to use the library and to make
		     changes to the source code, provided that the authors
		     name is left where it appears and that changes are
		     metioned in some notice. The library must be
		     distributed including sources, you are not allowed to
		     take money for this.
		     The parseAFM-package is copyrighted by Adobe Systems
		     Inc.
		     The type1 rasterizer is copyrighted by IBM and the
		     X11-consortium.
  ----- Warranties:  Of course, there's NO WARRANTY OF ANY KIND :-)
  ----- Credits:     I want to thank IBM and the X11-consortium for making
                     their rasterizer freely available.
		     Also thanks to Piet Tutelaers for his ps2pk, from
		     which I took the rasterizer sources in a format
		     independent from X11.
                     Thanks to all people who make free software living!
--------------------------------------------------------------------------*/
  
#define T1Y_C


#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>

#include "ylib_p.h"
#include "yshm.h"

#include "../type1/types.h"
#include "parseAFM.h" 
#include "../type1/objects.h" 
#include "../type1/spaces.h"  
#include "../type1/util.h" 
#include "../type1/fontfcn.h"

#include "t1types.h"
#include "t1extern.h"
#include "t1set.h"
#include "t1aaset.h"
#include "t1load.h"
#include "t1finfo.h"
#include "t1misc.h"
#include "t1ylib.h"
#include "t1base.h"

#define T1GCMASK GCForeground | GCBackground 


/* The three important X11 parameters t1lib has to deal with: */
static YDisplay *T1_display=NULL;  /* Must be accessible from t1delete.c */

/* The following parameter determines whether t1lib will use logical
   positioning of chars and string (place the origin at specified
   destination-point) or absolute positioning with respect to the
   origin (upper left corner) of the generated bitmap/pixmap. */
static int T1_lposition=1;

static YColor aacolors[AAMAXPLANES];
static YColor oldfg=0, oldbg=0;

/* T1_SetX11Params(): Set X11-parameters which t1lib has to know in order
   to properly generate pixmaps from characters */
int T1_SetYParams( YDisplay *display )

{
  T1_display = display;

  return(0);
}

		     
/* T1_SetCharY(): Generate an object of type GLYPH, i.e, a glyph with
   a pixmap ID instead of a pointer to a bitmap: */

GLYPH *T1_SetCharY( YGC *gc, int mode, int x_dest, int y_dest,
		    int FontID, unsigned char charcode,
		    float size, float angle)
{
  int i;

  GLYPH  *pglyph;
  static GLYPH xglyph={NULL,{0,0,0,0,0},NULL,0};
 
  int height, width;
  int opaque;
  

  xglyph.metrics.leftSideBearing=0;
  xglyph.metrics.rightSideBearing=0;
  xglyph.metrics.characterWidth=0;
  xglyph.metrics.ascent=0;
  xglyph.metrics.descent=0;
  xglyph.pFontCacheInfo=NULL;
  

  opaque=mode;
  
  /* First, check for a correct ID */
  i=CheckForFontID(FontID);
  if (i==-1)
    return(NULL);
  
  if ((pglyph=T1_SetChar( FontID, charcode, size, angle))==NULL){
    T1_PrintLog( "T1_SetCharY()",
		 "T1_SetChar() returned NULL-pointer!",
		 T1LOG_WARNING);
    return(NULL);
  }
  width=pglyph->metrics.rightSideBearing-pglyph->metrics.leftSideBearing;
  height=pglyph->metrics.ascent+pglyph->metrics.descent;

  {
    int x, y, i;
    unsigned char *ptr;
    int bit;
    int real_width = PAD(width, pFontBase->bitmap_pad);
    YImage *yimage;
    YGC *image_gc;
    YRect size;
    YPoint src, dest;
    YPoint *point = (YPoint*)malloc(sizeof(YPoint));
    YSharedWindowInfo *info = YSHM_getSharedWindowInfo(gc->window->shared_window);

    yimage = YCreateImage(T1_display,
			  real_width, height, info->background);

    image_gc = YGetGC(yimage);

    YSetForeground(image_gc, YGetColorRGB(255, 255, 255));

    y = 0;
    x = 0;

    ptr = (unsigned char*)pglyph->bits;
    bit = 0;

    while (y < height)
      {
	while (x < real_width / 8)
	  {
	    if ((*ptr & (1 << bit)) != 0)
	      {
		YPoint *point = (YPoint*)malloc(sizeof(YPoint));
		
		point->x = x * 8 + bit;
		point->y = y;

		YDrawPoints(image_gc, &point, 1);
		/*		printf ("X");*/
	      }
	    else
	      {
		/*printf (" ");*/
	      }

	    if (bit == 7)
	      {
		x ++;
		ptr++;
		bit = 0;
	      }
	    else
	      bit += 1;
	  }

	y++;
	x=0;
	/*	printf ("\n");*/
      }

    src.x = 0;
    src.y = 0;
    dest.x = x_dest;
    dest.y = y_dest;
    size.right = real_width;
    size.bottom = height;

    YWinBlit(gc, yimage, &src, &dest, &size);
  }

  /* Correct position */
  if (T1_lposition){
    x_dest += pglyph->metrics.leftSideBearing;
    y_dest -= pglyph->metrics.ascent;
  }

  pglyph->bits=NULL;    /* Since XDestroyImage() free's this also! */
  xglyph.metrics.leftSideBearing=pglyph->metrics.leftSideBearing;
  xglyph.metrics.rightSideBearing=pglyph->metrics.rightSideBearing;
  xglyph.metrics.characterWidth=pglyph->metrics.characterWidth;
  xglyph.metrics.ascent=pglyph->metrics.ascent;
  xglyph.metrics.descent=pglyph->metrics.descent;
  xglyph.bpp=pglyph->bpp;
  
  return( &xglyph);
}



/* T1_SetStringY(...): Draw a string of characters into an X11 drawable */
GLYPH *T1_SetStringY( YGC *gc, int mode, int x_dest, int y_dest,
		      int FontID, unsigned char *string, int len, 
		      long spaceoff, int kerning, float size, float angle)
{
  int i;
  GLYPH  *pglyph;
  static GLYPH xglyph={NULL,{0,0,0,0,0},NULL,0};

  int height, width;
  int opaque;
  

  xglyph.metrics.leftSideBearing=0;
  xglyph.metrics.rightSideBearing=0;
  xglyph.metrics.characterWidth=0;
  xglyph.metrics.ascent=0;
  xglyph.metrics.descent=0;
  xglyph.pFontCacheInfo=NULL;
  

  opaque=mode;
  
  /* First, check for a correct ID */
  i=CheckForFontID(FontID);
  if (i==-1)
    return(NULL);
  
  if ((pglyph=T1_SetString( FontID, string, len, 
			    spaceoff, kerning, size, angle))==NULL){
    T1_PrintLog( "T1_SetStringY()",
		 "T1_SetString() returned NULL-pointer!",
		 T1LOG_WARNING);
    return(NULL);
  }
  width=pglyph->metrics.rightSideBearing-pglyph->metrics.leftSideBearing;
  height=pglyph->metrics.ascent+pglyph->metrics.descent;

  {
    int x, y, i;
    unsigned char *ptr;
    int bit;
    int real_width = PAD(width, pFontBase->bitmap_pad);
    YImage *yimage;
    YGC *image_gc;
    YRect size;
    YPoint src, dest;
    YPoint *point = (YPoint*)malloc(sizeof(YPoint));
    YSharedWindowInfo *info = YSHM_getSharedWindowInfo(gc->window->shared_window);

    yimage = YCreateImage(T1_display,
			  real_width, height, info->background);

    YSHM_releaseSharedWindowInfo(info);

    image_gc = YGetGC(yimage);
    YSetForeground(image_gc, YGetColorRGB(255,255,255));

    y = 0;
    x = 0;

    ptr = (unsigned char*)pglyph->bits;
    bit = 0;

    while (y < height)
      {
	while (x < real_width / 8)
	  {
	    if ((*ptr & (1 << bit)) != 0)
	      {
		point->x = x * 8 + bit;
		point->y = y;

		YDrawPoints(image_gc, &point, 1);
		/*		printf ("X");*/
	      }
	    else
	      {
		/*printf (" ");*/
	      }

	    if (bit == 7)
	      {
		x ++;
		ptr++;
		bit = 0;
	      }
	    else
	      bit += 1;
	  }

	y++;
	x=0;
	/*	printf ("\n");*/
      }

    src.x = 0;
    src.y = 0;
    dest.x = x_dest;
    dest.y = y_dest;
    size.right = real_width;
    size.bottom = height;

    YWinBlit(gc, yimage, &src, &dest, &size);
  }
  
  /* Correct position */
  if (T1_lposition){
    x_dest += pglyph->metrics.leftSideBearing;
    y_dest -= pglyph->metrics.ascent;
  }

  pglyph->bits=NULL;    /* Since XDestroyImage() free's this also! */
  xglyph.metrics.leftSideBearing=pglyph->metrics.leftSideBearing;
  xglyph.metrics.rightSideBearing=pglyph->metrics.rightSideBearing;
  xglyph.metrics.characterWidth=pglyph->metrics.characterWidth;
  xglyph.metrics.ascent=pglyph->metrics.ascent;
  xglyph.metrics.descent=pglyph->metrics.descent;
  xglyph.bpp=pglyph->bpp;
  
  return( &xglyph);
}




/* T1_SetCharY(): Generate an object of type GLYPH, i.e, a glyph with
   a pixmap ID instead of a pointer to a bitmap: */

GLYPH *T1_AASetCharY( YGC *gc, int mode, int x_dest, int y_dest,
		      int FontID, unsigned char charcode,
		      float size, float angle)
{
  int i, j, k;
  YColor fg, bg;

  GLYPH  *pglyph;
  
  static GLYPH xglyph={NULL,{0,0,0,0,0},NULL,0};
 
  int height, width;
  
  int clipmask_h, clipmask_v, line_off;
  unsigned char *clipmask_ptr;
  
  int opaque;
  

  xglyph.metrics.leftSideBearing=0;
  xglyph.metrics.rightSideBearing=0;
  xglyph.metrics.characterWidth=0;
  xglyph.metrics.ascent=0;
  xglyph.metrics.descent=0;
  xglyph.pFontCacheInfo=NULL;
  

  opaque=mode;
  
  /* First, check for a correct ID */
  i=CheckForFontID(FontID);
  if (i==-1)
    return(NULL);
  
  /* In order to be able to contruct the pixmap we need to know
     foreground and background color as well the copy function */
  fg = gc->foreground;
  bg = gc->background;
  
  xglyph.bpp=32;
  
  /* At this point we must compute the colors that are needed to do
     antialiasing between fore- and background. The following function
     fills the static aacolors */
  if ((fg!=oldfg)||(bg!=oldbg)){
    T1_ComputeAAColorsY( fg, bg, AAMAXPLANES);
    T1_AASetGrayValues(aacolors[0],   /* white */
		       aacolors[1],
		       aacolors[2],
		       aacolors[3],
		       aacolors[4] ); /* black */
  }
  if ((pglyph=T1_AASetChar( FontID, charcode, size, angle))==NULL){
    T1_PrintLog( "T1_AASetCharY()",
		 "T1_AASetChar() returned NULL-pointer!",
		 T1LOG_WARNING);
    return(NULL);
  }
  width=pglyph->metrics.rightSideBearing-pglyph->metrics.leftSideBearing;
  height=pglyph->metrics.ascent+pglyph->metrics.descent;
  
#if 0
  if (opaque==0){
    clipmask_v=height;
    clipmask_h=width;
    clipmask_ptr=(unsigned char *)calloc( (PAD(clipmask_h, pFontBase->bitmap_pad)>>3)
					  * clipmask_v, sizeof( char));
    /* Note: We pad the clipmask always to byte boundary */
    if (pglyph->bpp==8)
      for ( k=0; k<clipmask_v; k++){
	line_off=k*(PAD(clipmask_h, 8)>>3);
	for (j=0; j<clipmask_h; j++){
	  if (((unsigned char *)(pglyph->bits))[k*width+j]!=bg)
	    clipmask_ptr[line_off+(j>>3)] |= (0x01<<(j%8));
	}
      }
    else if (pglyph->bpp==16)
      for ( k=0; k<clipmask_v; k++){
	line_off=k*(PAD(clipmask_h, 8)>>3);
	for (j=0; j<clipmask_h; j++){
	  if (((unsigned short *)(pglyph->bits))[k*width+j]!=bg)
	    clipmask_ptr[line_off+(j>>3)] |= (0x01<<(j%8));
	}
      }
    else 
      for ( k=0; k<clipmask_v; k++){
	line_off=k*(PAD(clipmask_h, 8)>>3);
	for (j=0; j<clipmask_h; j++){
	  if (((unsigned long *)(pglyph->bits))[k*width+j]!=bg)
	    clipmask_ptr[line_off+(j>>3)] |= (0x01<<(j%8));
	}
      }
    
    clipmask=XCreateBitmapFromData( T1_display,
				    d,
				    clipmask_ptr,
				    width,
				    height
				    );
    free( clipmask_ptr);
    XSetClipMask(T1_display, gc, clipmask);
    XSetClipOrigin(T1_display, gc, x_dest, y_dest);

  }
#endif

  /* Correct position */
  if (T1_lposition){
    x_dest += pglyph->metrics.leftSideBearing;
    y_dest -= pglyph->metrics.ascent;
  }

#if 0
  ximage=XCreateImage( T1_display,
		       T1_visual,
		       currdepth, 
		       ZPixmap, /* XYBitmap or XYPixmap */
		       0, /* No offset */
		       pglyph->bits,
		       width,
		       height,
		       pFontBase->bitmap_pad,  
		       0 /*PAD(width,8)/8*/  /* number of bytes per line */
		       );
  XPutImage(T1_display,
	    d,
	    gc,
	    ximage,
	    0,  
	    0,  
	    x_dest,  
	    y_dest,  
	    width,
	    height
	    );
  XDestroyImage(ximage);
  if (clipmask){
    XFreePixmap( T1_display, clipmask);
    clipmask=0;
    XSetClipMask(T1_display, gc, None);
    XSetClipOrigin(T1_display, gc, 0, 0);
  }
#endif

  {
    YImage *yimage;
    YSharedWindowInfo *info = YSHM_getSharedWindowInfo(gc->window->shared_window);
    YRect size;
    YPoint src, dest;

    yimage = YCreateImage(T1_display,
			  width, height,
			  info->background);

    YSHM_releaseSharedWindowInfo(info);

    memcpy(yimage->image_data, pglyph->bits, width * height * sizeof(YColor));

    src.x = 0;
    src.y = 0;
    dest.x = x_dest;
    dest.y = y_dest;
    size.right = width;
    size.bottom = height;

    YWinBlit(gc, yimage, &src, &dest, &size);
  }
  
  pglyph->bits=NULL;    /* Since XDestroyImage() free's this also! */
  xglyph.metrics.leftSideBearing=pglyph->metrics.leftSideBearing;
  xglyph.metrics.rightSideBearing=pglyph->metrics.rightSideBearing;
  xglyph.metrics.characterWidth=pglyph->metrics.characterWidth;
  xglyph.metrics.ascent=pglyph->metrics.ascent;
  xglyph.metrics.descent=pglyph->metrics.descent;
  xglyph.bpp=pglyph->bpp;
  
  return( &xglyph);
}






/* T1_SetStringY(...): Draw a string of characters into an X11 drawable */
GLYPH *T1_AASetStringY( YGC *gc, int mode, int x_dest, int y_dest,
			int FontID, unsigned char *string, int len, 
			long spaceoff, int kerning, float size, float angle)
{
  int i, j, k;

  GLYPH  *pglyph;
#if 0
  XImage *ximage;
#endif
  
static GLYPH xglyph={NULL,{0,0,0,0,0},NULL,0};
 
  int height, width;
  
  YColor fg, bg;
#if 0
  XGCValues xgcvalues;

  Pixmap clipmask=0;
  int clipmask_h, clipmask_v, line_off;
  unsigned char *clipmask_ptr;
  
#endif  
  int opaque;

  xglyph.metrics.leftSideBearing=0;
  xglyph.metrics.rightSideBearing=0;
  xglyph.metrics.characterWidth=0;
  xglyph.metrics.ascent=0;
  xglyph.metrics.descent=0;
  xglyph.pFontCacheInfo=NULL;
  

  opaque=mode;
  
  /* First, check for a correct ID */
  i=CheckForFontID(FontID);
  if (i==-1)
    return(NULL);
  
  /* In order to be able to contruct the pixmap we need to know
     foreground and background color as well the copy function */
  fg = gc->foreground;
  bg = gc->background;

  xglyph.bpp=32;
  
  /* At this point we must compute the colors that are needed to do
     antialiasing between fore- and background. The following function
     fills the static aacolors */
  if ((fg!=oldfg)||(bg!=oldbg)){
    T1_ComputeAAColorsY( fg, bg, AAMAXPLANES);
    T1_AASetGrayValues(aacolors[0],   /* white */
		       aacolors[1],
		       aacolors[2],
		       aacolors[3],
		       aacolors[4] ); /* black */
  }
  if ((pglyph=T1_AASetString( FontID, string, len, 
			      spaceoff, kerning, size, angle))==NULL){
    T1_PrintLog( "T1_AASetStringY()",
		 "T1_AASetString() returned NULL-pointer!",
		 T1LOG_WARNING);
    return(NULL);
  }
  width=pglyph->metrics.rightSideBearing-pglyph->metrics.leftSideBearing;
  height=pglyph->metrics.ascent+pglyph->metrics.descent;

#if 0
  if (opaque==0){
    clipmask_v=height;
    clipmask_h=width;
    clipmask_ptr=(unsigned char *)calloc( (PAD(clipmask_h, pFontBase->bitmap_pad)>>3)
					  * clipmask_v, sizeof( char));
    /* Note: We pad the clipmask always to byte boundary */
    if (pglyph->bpp==8)
      for ( k=0; k<clipmask_v; k++){
	line_off=k*(PAD(clipmask_h, 8)>>3);
	for (j=0; j<clipmask_h; j++){
	  if (((unsigned char *)(pglyph->bits))[k*width+j]!=bg)
	    clipmask_ptr[line_off+(j>>3)] |= (0x01<<(j%8));
	}
      }
    else if (pglyph->bpp==16)
      for ( k=0; k<clipmask_v; k++){
	line_off=k*(PAD(clipmask_h, 8)>>3);
	for (j=0; j<clipmask_h; j++){
	  if (((unsigned short *)(pglyph->bits))[k*width+j]!=bg)
	    clipmask_ptr[line_off+(j>>3)] |= (0x01<<(j%8));
	}
      }
    else 
      for ( k=0; k<clipmask_v; k++){
	line_off=k*(PAD(clipmask_h, 8)>>3);
	for (j=0; j<clipmask_h; j++){
	  if (((unsigned long *)(pglyph->bits))[k*width+j]!=bg)
	    clipmask_ptr[line_off+(j>>3)] |= (0x01<<(j%8));
	}
      }
    
    clipmask=XCreateBitmapFromData( T1_display,
				    d,
				    clipmask_ptr,
				    width,
				    height
				    );
    free( clipmask_ptr);
    XSetClipMask(T1_display, gc, clipmask);
    XSetClipOrigin(T1_display, gc, x_dest, y_dest);

  }
#endif
  
  /* Correct position */
  if (T1_lposition){
    x_dest += pglyph->metrics.leftSideBearing;
    y_dest -= pglyph->metrics.ascent;
  }
#if 0
  ximage=XCreateImage( T1_display,
		       T1_visual,
		       currdepth, 
		       ZPixmap, /* XYBitmap or XYPixmap */
		       0, /* No offset */
		       pglyph->bits,
		       width,
		       height,
		       pFontBase->bitmap_pad,  /* lines padded to bytes */
		       0 /*PAD(width,8)/8*/  /* number of bytes per line */
		       );
  XPutImage(T1_display,
	    d,
	    gc,
	    ximage,
	    0,  
	    0,  
	    x_dest,  
	    y_dest,  
	    width,
	    height
	    );
  XDestroyImage(ximage);
  if (clipmask){
    XFreePixmap( T1_display, clipmask);
    clipmask=0;
    XSetClipMask(T1_display, gc, None);
    XSetClipOrigin(T1_display, gc, 0, 0);
  }
#endif

  {
    YImage *yimage;
    YSharedWindowInfo *info = YSHM_getSharedWindowInfo(gc->window->shared_window);
    YRect size;
    YPoint src, dest;

    yimage = YCreateImage(T1_display,
			  width, height,
			  info->background);

    YSHM_releaseSharedWindowInfo(info);

    memcpy(yimage->image_data, pglyph->bits, width * height * sizeof(YColor));

    src.x = 0;
    src.y = 0;
    dest.x = x_dest;
    dest.y = y_dest;
    size.right = width;
    size.bottom = height;

    YWinBlit(gc, yimage, &src, &dest, &size);
  }
  
  pglyph->bits=NULL;    /* Since XDestroyImage() free's this also! */
  xglyph.metrics.leftSideBearing=pglyph->metrics.leftSideBearing;
  xglyph.metrics.rightSideBearing=pglyph->metrics.rightSideBearing;
  xglyph.metrics.characterWidth=pglyph->metrics.characterWidth;
  xglyph.metrics.ascent=pglyph->metrics.ascent;
  xglyph.metrics.descent=pglyph->metrics.descent;
  xglyph.bpp=pglyph->bpp;
  
  return( &xglyph);
}



/* T1_ComputeAAColorsY(): Compute the antialiasing colors in dependency
   of foreground and background */
int T1_ComputeAAColorsY( YColor fg, YColor bg, int nolevels)
{
  static YColor last_fg;
  static YColor last_bg;
  unsigned long delta_red, delta_green, delta_blue;
  int i;
  int nocolors=0;
  uint8 fg_red, fg_blue, fg_green;
  uint8 bg_red, bg_blue, bg_green;

  aacolors[0]=bg;
  aacolors[nolevels-1]=fg;

  if ((fg==last_fg)&&(bg==last_bg))
    return(nocolors);
  
  /* Get RGB values for fore- and background */
  YColorGetRGB(fg, &fg_red, &fg_green, &fg_blue);
  YColorGetRGB(bg, &bg_red, &bg_green, &bg_blue);

  delta_red   = (fg_red - bg_red)/4;
  delta_green = (fg_green - bg_green)/4;
  delta_blue  = (fg_blue - bg_blue)/4;
  
  for (i=1; i<nolevels-1; i++){
    uint8 prev_red, prev_green, prev_blue;
    YColorGetRGB(aacolors[i-1], &prev_red, &prev_green, &prev_blue);

    aacolors[i] = YGetColorRGB(prev_red + delta_red,
			       prev_green + delta_green,
			       prev_blue + delta_blue);
    nocolors++;
  }
  
  return(nocolors);
}


/* Set the positioning switch */
void T1_LogicalPositionY( int pos_switch)
{
  if (pos_switch)
    T1_lposition=1;
  else
    T1_lposition=0;
  return;
}

