/* -*- Mode: C; tab-width: 4 -*-
   yfonts_p.h -- internal stuff dealing with fonts and font renderers.
   Created: Chris Toshok <toshok@hungry.com>, 23-Sep-1997
*/

#ifndef _y_fonts_p_h
#define _y_fonts_p_h

#include "yfonts.h"

typedef enum {
  YFONT_BOLD,
  YFONT_MEDIUM,
  YFONT_LIGHT,
} YFontWeight;

typedef enum {
  YFONT_REGULAR,
  YFONT_OBLIQUE,
  YFONT_ITALIC
} YFontSlant;

typedef struct YFontRenderer_vtable
{
  const char* (*getName)(YFontRenderer *);
  const char* (*getVendor)(YFontRenderer *);
  const char* (*getVersion)(YFontRenderer *);

  YFont* (*loadFont)(YFontRenderer *,
					 const char *font_spec);

  int (*countFonts)(YFontRenderer *);
  int (*listFonts)(YFontRenderer *,
				   const char ***font_specs);

  void (*renderCharacter)(YFontRenderer *,
						  char *string, int length,
						  YGC *gc,
						  YPoint *start_at);

  int (*maxAscent)(YFontRenderer *, YFont *font);
  int (*maxDescent)(YFontRenderer *, YFont *font);

  int (*maxWidth)(YFontRenderer *, YFont *font);

  int (*advancement)(YFontRenderer *, YFont *font, char character);
  int (*stringWidth)(YFontRenderer *, YFont *font, char *string, int string_length);

  void (*destroy)(YFontRenderer *);

} YFontRenderer_vtable;

struct YFontRenderer_struct
{
  YFontRenderer_vtable _vtable;

  void *renderer_data;
};

struct YFont_struct
{
  YFontRenderer *renderer;

  char *font_spec;

  void *font_data;
};

/* This function is implemented in each of the respective font renderers
   and is responsible for sending back the pointer to the font renderer */
extern YFontRenderer *YFontRenderer_get();

extern int YFontRenderer_countFonts(YFontRenderer *renderer);
extern int YFontRenderer_listFonts(YFontRenderer *renderer, const char ***font_specs);
extern YFont *YFontRenderer_loadFont(YFontRenderer *renderer,
									 const char *spec);
extern void YFontRenderer_renderCharacter(YFontRenderer *renderer,
										  char *string, int length,
										  YGC *gc, YPoint *start_at);

extern int YFontRenderer_maxAscent(YFontRenderer *renderer, YFont *font);
extern int YFontRenderer_maxDescent(YFontRenderer *renderer, YFont *font);
extern int YFontRenderer_maxWidth(YFontRenderer *renderer, YFont *font);
extern int YFontRenderer_advancement(YFontRenderer *renderer,
									 YFont *font,
									 char character);
extern int YFontRenderer_stringWidth(YFontRenderer *renderer,
									 YFont *font,
									 char *string, int length);
extern void YFontRenderer_destroy(YFontRenderer *renderer);

extern void Y_registerFontRenderers();
YFontRenderer *Y_GetFontRendererByName (char *name);


#endif /* _y_fonts_p_h */
