/* -*- Mode: C; tab-width: 4 -*-
   dihw.c -- device independent hardware routines.
   Created: Chris Toshok <toshok@hungry.com>, 14-Oct-1997
*/

/* So, this is home of incompletely, inefficient implementations.  Don't
   go near this code unless you want to speed things up. */

#include <assert.h>
#include "yshm.h"
#include "ylib_p.h"
#include "hw.h"
#include "dihw.h"

void 
YDIHW_DrawRectangle(YHW *hw, 
					YGC *gc, 
					int32 x, 
					int32 y, 
					uint32 width, 
					uint32 height)
{
  YDIHW_DrawLine(hw, gc, x, y, x + width, y);
  YDIHW_DrawLine(hw, gc, x + width, y, x + width, y + height);
  YDIHW_DrawLine(hw, gc, x, y + height, x + width, y +height);
  YDIHW_DrawLine(hw, gc, x, y, x, y + height);
}

void 
YDIHW_FillRectangle(YHW *hw, 
					YGC *gc, 
					int32 x, 
					int32 y, 
					uint32 width, 
					uint32 height)
{
  int y_cur;

  for (y_cur = y;
	   y_cur < y + height;
	   y_cur ++)
	{
	  (*hw->vtable.DrawSpan)(hw, gc, x, x + width, y_cur);
	}
}

static void
draw_vertical_line(YHW *hw,
				   YGC *gc,
				   int32 x1,
				   int32 y1,
				   int32 x2,
				   int32 y2)
{
  int x = x1;
  int y = y1;
  
  while (y < y2)
	{
	  /* XXX line thickness?  we only draw one pixel lines here. */
	  (*hw->vtable.DrawSpan)(hw, gc, x, x, y);
	  y++;
	}
}

static void
draw_horizontal_line(YHW *hw,
					 YGC *gc,
					 int32 x1,
					 int32 y1,
					 int32 x2,
					 int32 y2)
{
  /* XXX line thickness?  we only draw one pixel lines here. */
  (*hw->vtable.DrawSpan)(hw, gc, x1, x2, y1);
}

static void
draw_midpoint_line(YHW *hw,
				   YGC *gc,
				   int32 x1,
				   int32 y1,
				   int32 x2,
				   int32 y2)
{
  int x;
  double dy = y2 - y1;
  double dx = x2 - x1;
  double m = dy / dx;
  double y = y1;
  
  for (x = x1; x <= x2; x ++)
	{
	  /* XXX line thickness?  we only draw one pixel lines here. */
	  (*hw->vtable.DrawSpan)(hw, gc, x, x, y);
	  y += m;
	}
}

void 
YDIHW_DrawLine(YHW *hw, 
			   YGC *gc, 
			   int32 x1, 
			   int32 y1, 
			   int32 x2, 
			   int32 y2)
{
  if (x1 == x2)
	{
	  if (y1 > y2)
		{
		  int32 tmp = y1;
		  y1 = y2;
		  y2 = tmp;
		}

	  draw_vertical_line(hw, gc, x1, y1, x2, y2);
	}
  else if (y1 == y2)
	{
	  if (x1 > x2)
		{
		  int32 tmp = x1;
		  x1 = x2;
		  x2 = tmp;
		}

	  draw_horizontal_line(hw, gc, x1, y1, x2, y2);
	}
  else
	{
	  if (x1 > x2)
		{
		  int32 tmp = x1;
		  x1 = x2;
		  x2 = tmp;

		  tmp = y1;
		  y1 = y2;
		  y2 = tmp;
		}

	  draw_midpoint_line(hw, gc, x1, y1, x2, y2);
	}
}

void
YDIHW_DrawPoints(YHW *hw,
				 YGC *gc,
				 YPoint **points,
				 int npoints)
{
  int i;

  for (i = 0; i < npoints; i ++)
	(*hw->vtable.DrawSpan)(hw, gc,
						   points[i]->x, points[i]->x, points[i]->y);
}

void 
YDIHW_DrawBezier(YHW *hw, 
				 YGC *gc, 
				 YPoint **points, 
				 int npoints)
{
  assert(0);
}
