/* -*- Mode: C; tab-width: 4 -*-
   hwmessages.h -- message types for communicating between the client and
                   hw_helper app.
   Created: Chris Toshok <toshok@hungry.com>, 27-Oct-1997
 */

#ifndef _y_hwmessages_h
#define _y_hwmessages_h

enum {
  YHW_GET_SIZE,

  YHW_INIT_DISPLAY,
  YHW_DEINIT_DISPLAY,
  YHW_OPEN_DISPLAY,
  YHW_CLOSE_DISPLAY,
  YHW_HANDLE_EVENTS,

  YHW_SET_CURSOR,
  YHW_MOVE_CURSOR,
  YHW_SHOW_CURSOR,
  YHW_HIDE_CURSOR,
  
  YHW_DRAW_RECTANGLE,
  YHW_FILL_RECTANGLE,
  YHW_DRAW_POINTS,
  YHW_DRAW_LINE,
  YHW_DRAW_ARC,
  YHW_DRAW_BEZIER,
  YHW_BLIT,

  YHW_CAN_DO_DPMS,
  YHW_HANDLE_DPMS
};

typedef struct {
  int32 tag;
  int32 driver_num;
} YHWGetSizeReq;

typedef struct {
  int32 tag;
  uint32 width;
  uint32 height;
} YHWGetSizeRep;

typedef struct {
  int32 tag;
} YHWInitDisplayReq;

typedef struct {
  int32 tag;
} YHWDeinitDisplayReq;

typedef struct {
  int32 tag;
} YHWOpenDisplayReq;

typedef struct {
  int32 tag;
} YHWCloseDisplayReq;

typedef struct {
  int32 tag;
  int32 image_shmid;
  int32 hot_x;
  int32 hot_y;
} YHWSetCursorReq;

typedef struct {
  int32 tag;
  int32 x;
  int32 y;
} YHWMoveCursorReq;

typedef struct {
  int32 tag;
} YHWShowCursorReq;

typedef struct {
  int32 tag;
} YHWHideCursorReq;

typedef struct {
  int32 tag;
  int32 gc_shmid;
  int32 x;
  int32 y;
  int32 width;
  int32 height;
} YHWDrawRectangleReq;

typedef struct {
  int32 tag;
  int32 gc_shmid;
  int32 x;
  int32 y;
  int32 width;
  int32 height;
} YHWFillRectangleReq;

typedef struct { /* many of these may need to be sent for a give call to YHW_DrawPoints */
  int32 tag;
  int32 gc_shmid;
  YPoint points[20];
  int32 npoints;
} YHWDrawPointsReq;

typedef struct {
  int32 tag;
  int32 gc_shmid;
  int32 x1;
  int32 y1;
  int32 x2;
  int32 y2;
} YHWDrawLineReq;

typedef struct {
  int32 tag;
  int32 gc_shmid;
  int32 x;
  int32 y;
  int32 width;
  int32 height;
} YHWDrawArcReq;

/* many of these many need to be sent for a given call to YHW_DrawBezier.  total_points gives the total
   number of points for the given bezier, and should be used to determine if more of these requests are
   on the way. */
typedef struct {
  int32 tag;
  int32 gc_shmid;
  int32 total_points;
  YPoint points[20];
  int32 npoints;
} YHWDrawBezierReq;

typedef struct {
  int32 tag;
  int32 gc_shmid;
  int32 src_shmid;
  int32 dest_shmid;
  int32 src_x;
  int32 src_y;
  int32 dest_x;
  int32 dest_y;
  int32 width;
  int32 height;
} YHWBlitReq;

typedef struct {
  int32 tag;
} YHWCanDoDPMSReq;

typedef struct {
  int32 tag;
  YBool can_do;
} YHWCanDoDPMSRep;

typedef struct {
  int32 tag;
  int32 dpms_mode;
} YHWHandleDPMSReq;

typedef union {
  int32 tag;
  YHWGetSizeReq get_size;
  YHWInitDisplayReq init_display;
  YHWDeinitDisplayReq deinit_display;
  YHWOpenDisplayReq open_display;
  YHWCloseDisplayReq close_display;
  YHWSetCursorReq set_cursor;
  YHWMoveCursorReq move_cursor;
  YHWShowCursorReq show_cursor;
  YHWHideCursorReq hide_cursor;
  YHWDrawRectangleReq draw_rectangle;
  YHWFillRectangleReq fill_rectangle;
  YHWDrawPointsReq draw_points;
  YHWDrawLineReq draw_line;
  YHWDrawArcReq draw_arc;
  YHWDrawBezierReq draw_bezier;
  YHWBlitReq blit;
  YHWCanDoDPMSReq cando_dpms;
  YHWHandleDPMSReq handle_dpms;
} YHWMessage;

typedef union 
{
  int32 tag;
  YHWGetSizeRep get_size;
  YHWCanDoDPMSRep cando_dpms;
} YHWReply;

#endif /* _y_hwmessages_h */
