/* -*- Mode: C; tab-width: 4 -*-
   ydisplay.c -- YDisplay methods.
   Created: Chris Toshok <toshok@hungry.com>, 8-May-1997
 */
/* 
   Copyright (C) 1996, 1997 The Hungry Programmers.
   
   This file is part of The Y Window System.
   
   Y is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
   
   Y is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.
   
   You should have received a copy of the GNU General Public License
   along with GLD; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307 , USA.
*/

#include "ylib_p.h"
#include "yfonts_p.h"
#include "ymessages.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* the vtables for local and network displays */
extern YDisplay_vtable yLocalDisplay_vtable;
extern YDisplay_vtable yNetDisplay_vtable;
extern YDisplay_vtable yMemDisplay_vtable;

YDisplay *
YOpenDisplay(char *display_name)
{
  YDisplay *display = (YDisplay *)calloc(1, sizeof(YDisplay));

  if (display_name == NULL) 
    display_name = getenv("YDISPLAY");

  if (display_name == NULL)
	display_name = ":0";

  if (!strcmp(display_name, ":0"))
	display->vtable = yLocalDisplay_vtable;
  else if (!strcmp(display_name, "<mem>"))
	display->vtable = yMemDisplay_vtable;
  else
	display->vtable = yNetDisplay_vtable;

  display->root = (YWindow*)0xbeefbabe;
  
  display->display_name = strdup(display_name);

  /* initialize the font machinery. */
  Y_registerFontRenderers();

  if (display->vtable.OpenConnection)
	(*display->vtable.OpenConnection)(display);

  return display;
}

void
YCloseDisplay(YDisplay *display)
{
  if (display->vtable.CloseConnection)
	(*display->vtable.CloseConnection)(display);
  free(display);
  /* more? */
}

int
YListFonts(YDisplay *display,
		   const char **font_specs)
{
  if (display->vtable.ListFonts)
	return (*display->vtable.ListFonts)(display, font_specs);
  else
	return 0;
}

int
YGetDisplayFD(YDisplay *display)
{
  if (display->vtable.GetConnectionFD)
	return (*display->vtable.GetConnectionFD)(display);
  else
	return -1;
}

YWindow *
YRootWindowOfDisplay(YDisplay *display)
{
  return display->root;
}

void
YGetNextEvent(YDisplay *display, Y_Event *event)
{
  if (display->vtable.GetNextEvent)
	(*display->vtable.GetNextEvent)(display, event);
}

void
Y_handleEvent(Y_Event *event)
{
  printf ("Y_handleEvent() called in client.\n");
}
