/* -*- Mode: C; tab-width: 4 -*-
   ydspmem.c -- Y window system memory display connections.
   Created: Chris Toshok <toshok@hungry.com>, 21-Sep-1997
 */
/* 
   Copyright (C) 1996, 1997 The Hungry Programmers.
   
   This file is part of The Y Window System.
   
   Y is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
   
   Y is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.
   
   You should have received a copy of the GNU General Public License
   along with GLD; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307 , USA.
*/

#include "ylib_p.h"
#include "yshm.h"
#include "ymessages.h"
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

extern YWindow_vtable yMemWindow_vtable;

static YWindow*
MemCreateWindow(YDisplay *display,
				  YWindow *parent_window,
				  int x, int y, int width, int height,
				  YColor bg_color)
{
  YWindow *new_window;
  YSharedWindowInfo *info;
  YSharedWindowInfo *parent_info;
  int parent_sem, sem;

  /*  D(("Inside MemCreateWindow()\n"));*/

  new_window = (YWindow*)calloc(1, sizeof(YWindow));

  new_window->display = display;
  new_window->vtable = yMemWindow_vtable;

  new_window->shared_window = YSHM_allocateSharedWindowInfo();
  info = YSHM_getSharedWindowInfo(new_window->shared_window);

  sem = YSHM_lockSemaphore(info->sem_id);

  memset(info, 0, sizeof(YSharedWindowInfo));

  info->bounds.left = 0;
  info->bounds.top = 0;
  info->bounds.right = width;
  info->bounds.bottom = height;

  if (parent_window != (YWindow*)0xbeefbabe)
	{
	  info->parent_id = parent_window->shared_window;
	  
	  /* Now we have to lock the parent's semaphore and add the
		 new window to its list of children. */
	  parent_info = YSHM_getSharedWindowInfo(parent_window->shared_window);
	  parent_sem = YSHM_lockSemaphore(parent_info->sem_id);
	  
	  info->prev_sibling = 0;
	  info->next_sibling = parent_info->top_child;
	  
	  if(info->next_sibling != 0)
		{
		  YSharedWindowInfo *sib_info;
		  int sib_sem;
		  
		  sib_info = YSHM_getSharedWindowInfo(info->next_sibling);
		  sib_sem = YSHM_lockSemaphore(sib_info->sem_id);
		  
		  sib_info->prev_sibling = new_window->shared_window;
		  
		  YSHM_unlockSemaphore(sib_info->sem_id, sib_sem);
		  YSHM_releaseSharedWindowInfo(sib_info);
		}
	  
	  parent_info->top_child = new_window->shared_window;
	  
	  if (parent_info->bottom_child == 0)
		parent_info->bottom_child = new_window->shared_window;
	}

  if (parent_window != (YWindow*)0xbeefbabe)
	{
	  YSHM_unlockSemaphore(parent_info->sem_id, parent_sem);

	  YSHM_releaseSharedWindowInfo(parent_info);
	}

  YSHM_unlockSemaphore(info->sem_id, sem);
  YSHM_releaseSharedWindowInfo(info);
  
  return new_window;
}

static void
MemDestroyWindow(YDisplay *display, YWindow *window)
{
  YSharedWindowInfo *info;

  D(("Inside MemDestroyWindow()\n"));

  info = YSHM_getSharedWindowInfo(window->shared_window);

  YSHM_releaseSharedWindowInfo(info);

  YSHM_destroySharedWindowInfo(window->shared_window);

  free(window);
}

static YWindow *
MemGetWindowById(YDisplay *display, YWindow_id id)
{
  YWindow *new_window = (YWindow*)calloc(1, sizeof(YWindow));

  new_window->shared_window = id;
  new_window->display = display;
  new_window->vtable = yMemWindow_vtable;

  return new_window;
}

YDisplay_vtable yMemDisplay_vtable = {
  NULL,
  NULL,
  NULL,
  MemCreateWindow,
  MemDestroyWindow,
  MemGetWindowById,
  NULL
};
