/* -*- Mode: C; tab-width: 4 -*-
   ygc.c -- Y window system graphics contexts.
   Created: Chris Toshok <toshok@hungry.com>, 8-May-1997
 */
/* 
   Copyright (C) 1996, 1997 The Hungry Programmers.
   
   This file is part of The Y Window System.
   
   Y is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
   
   Y is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.
   
   You should have received a copy of the GNU General Public License
   along with GLD; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307 , USA.
*/

#include "ylib_p.h"

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

/* the vtables for local and network rendering. */
extern YGC_vtable yLocalGC_vtable;
extern YGC_vtable yNetGC_vtable;

YGC *
YGetGC(YWindow *window)
{
  /*  D(("Inside YGetGC\n"));*/
  return (*window->vtable.GetGC)(window);
}

YGC *
YGetGCById(YGC_id gc_id)
{
}

void
YDestroyGC(YGC *gc)
{
  /* XXX - more needed ? */
  free(gc);
}

void
YSetForeground(YGC *gc, YColor foreground)
{
  (*gc->vtable.SetForeground)(gc, foreground);
}

void
YSetBackground(YGC *gc, YColor background)
{
  (*gc->vtable.SetBackground)(gc, background);
}

void
YSetFont(YGC *gc, YFont *font)
{
  assert(NULL != gc);
  assert(NULL != font);
  (*gc->vtable.SetFont)(gc, font);
}

void
YDrawLine(YGC *gc, YPoint *p1, YPoint *p2)
{
  (*gc->vtable.DrawLine)(gc, p1, p2);
}

void
YDrawPoints(YGC *gc,
			YPoint **points,
			int npoints)
{
  (*gc->vtable.DrawPoints)(gc, points, npoints);
}

void
YDrawRectangle(YGC *gc, YRect *rect)
{
  (*gc->vtable.DrawRect)(gc, rect);
}

void
YFillRectangle(YGC *gc, YRect *rect)
{
  (*gc->vtable.FillRect)(gc, rect);
}

void
YDrawAsciiText(YGC *gc, YPoint *where, char *ascii_text)
{
  (*gc->vtable.DrawAsciiText)(gc, where, ascii_text);
}

void
YWinBlit(YGC *gc, YWindow *source,
		 YPoint *src_loc, YPoint *dest_loc, YRect *size)
{
  if (gc->window->display == source->display) /* if they're on the same display. */
	{
	  (*gc->vtable.CopyFromWinToWin)(gc, source,
									 src_loc, dest_loc, size);
	}
  else
	{
	  YColor *color_buffer;
	  YGC *source_gc = YGetGC(source);

	  (*source_gc->vtable.CopyFromWin)(source_gc, src_loc, size, &color_buffer);

	  if (!color_buffer)
		{
		  /* do something spiffier than just return. XXX */
		  YDestroyGC(source_gc);
		  return;
		}

	  (*gc->vtable.CopyToWin)(gc, dest_loc, size, color_buffer);

	  YDestroyGC(source_gc);
	  free(color_buffer);
	}
}
