/* -*- Mode: C; tab-width: 4 -*-
   ygclocal.c -- Y window system graphics contexts, rendering to local
                 hardware.
   Created: Chris Toshok <toshok@hungry.com>, 8-May-1997
 */
/* 
   Copyright (C) 1996, 1997 The Hungry Programmers.
   
   This file is part of The Y Window System.
   
   Y is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
   
   Y is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.
   
   You should have received a copy of the GNU General Public License
   along with GLD; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307 , USA.
*/

#include "ylib_p.h"
#include "yfonts_p.h"
#include "yshm.h"
#include "hw.h"

#include <stdio.h>
#include <stdlib.h>

static void
LocalCopyFromWin(YGC *gc,
				 YPoint *src_loc, YRect *size, YColor **buffer)
{
  YHW_CopyFromWin(gc, src_loc->x, src_loc->y,
				  size->right, size->bottom, buffer);
}

static void
LocalCopyToWin(YGC *gc,
			   YPoint *dest_loc, YRect *size, YColor *buffer)
{
  YHW_CopyToWin(gc, dest_loc->x, dest_loc->y,
				size->right, size->bottom, buffer);
}

static void
LocalCopyFromWinToWin(YGC *gc,
					  YWindow *source_window, YPoint *src_loc, YPoint *dest_loc,
					  YRect *size)
{
  YHW_CopyFromWinToWin(gc, source_window,
					   src_loc->x, src_loc->y,
					   dest_loc->x, dest_loc->y,
					   size->right, size->bottom);
}

static void
LocalDrawLine(YGC *gc,
			  YPoint *p1,
			  YPoint *p2)
{
  YHW_DrawLine(gc,
			   p1->x, p1->y,
			   p2->x, p2->y);
}

static void
LocalDrawAsciiText(YGC *gc,
				   YPoint *where,
				   char *ascii_text)
{
  char c;
  YFont *font = gc->font;

  YFontRenderer_renderCharacter(font->renderer,
								ascii_text, strlen(ascii_text),
								gc, where);
}

static void
LocalDrawPoints(YGC *gc,
				YPoint **points,
				int npoints)
{
  YHW_DrawPoints(gc, points, npoints);
}

static void
LocalDrawRect(YGC *gc,
			  YRect *rect)
{
  YHW_DrawRectangle(gc,
					rect->left,
					rect->top,
					rect->right - rect->left,
					rect->bottom - rect->top);
}

static void
LocalFillRect(YGC *gc,
			  YRect *rect)
{
  YHW_FillRectangle(gc,
					rect->left,
					rect->top,
					rect->right - rect->left,
					rect->bottom - rect->top);
}

static void
LocalSetForeground(YGC *gc,
				   YColor foreground)
{
  gc->foreground = foreground;
}

static void
LocalSetBackground(YGC *gc,
				   YColor background)
{
  gc->background = background;
}

static void
LocalSetFont(YGC *gc,
			 YFont *font)
{
  gc->font = font;
}

static void
LocalDestroy(YGC *gc)
{
  /* free things. */
  free(gc);
}

YGC_vtable yLocalGC_vtable = {
  LocalCopyFromWin,
  LocalCopyToWin,
  LocalCopyFromWinToWin,
  LocalDrawLine,
  LocalDrawAsciiText,
  LocalDrawPoints,
  LocalDrawRect,
  LocalFillRect,
  LocalSetForeground,
  LocalSetBackground,
  LocalSetFont,
  LocalDestroy
};
