/* -*- Mode: C; tab-width: 4 -*-
   yimage.c -- Y window system offscreen images.
   Created: Chris Toshok <toshok@hungry.com>, 21-Sep-1997
 */
/* 
   Copyright (C) 1996, 1997 The Hungry Programmers.
   
   This file is part of The Y Window System.
   
   Y is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
   
   Y is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.
   
   You should have received a copy of the GNU General Public License
   along with GLD; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307 , USA.
*/

#include "ylib_p.h"
#include "yshm.h"

#if 0
#include "png.h"
#endif

#include <stdio.h>
#include <stdlib.h>

static YDisplay *offscreen_display;

YImage *
YCreateImage(YDisplay *display,
			 int width, int height,
			 YColor bg_color)
{
  YWindow *new_window;
  char name[200];

  if (!offscreen_display)
	offscreen_display = YOpenDisplay("<mem>");

  new_window = YCreateWindow(offscreen_display,
							 YRootWindowOfDisplay(offscreen_display),
							 0, 0, width, height, bg_color);

  snprintf (name, sizeof(name), "%d-image-data", new_window->shared_window);

  new_window->image_data =
	YSHM_allocateSharedMemory(name, width * height * sizeof(YColor));

  if (bg_color != 0)
	{
	  YColor *color_data = (YColor*)new_window->image_data;
	  int length = width * height;
	  int i;

	  for (i = 0; i < length; i ++)
		*(color_data ++) = bg_color;
	}

  return new_window;
}

void
YDestroyImage(YImage *image)
{
  char name[200];
  YSharedWindowInfo *info = YSHM_getSharedWindowInfo(image->shared_window);
  int width = info->bounds.right - info->bounds.left;
  int height = info->bounds.bottom - info->bounds.top;

  snprintf(name, sizeof(name), "%d-image-data", image->shared_window);

  YSHM_releaseSharedMemory(image->image_data, width * height * sizeof(YColor));

  YSHM_destroySharedMemory(name);

  YDestroyWindow((YWindow*)image);
}

#if 0
static void
ypng_error_fn(png_structp png_ptr, png_const_charp msg)
{
}

static void
ypng_warning_fn(png_structp png_ptr, png_const_charp msg)
{
}

YImage *
YCreateImageFromPNGFile(YDisplay *display,
						char *filename)
{
  png_structp png_ptr;
  png_infop info_ptr;
  png_uint_32 width, height;
  int bit_depth, color_type, interlace_type;
  FILE *fp;
  YImage *image;

  if ((fp = fopen(filename, "rb")) == NULL)
	return NULL;

  png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,
								   NULL,
								   ypng_error_fn,
								   ypng_warning_fn);

  if (png_ptr == NULL)
	{
	  fclose(fp);
	  return NULL;
	}

  /* Allocate/initialize the memory for image information */
  info_ptr = png_create_info_struct(png_ptr);
  if (info_ptr == NULL)
	{
	  fclose(fp);
	  png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
	  return NULL;
	}

  if (setjmp(png_ptr->jmpbuf))
	{
	  png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
	  fclose(fp);
	  return NULL;
	}
  
  png_init_io(png_ptr, fp);

  png_read_info(png_ptr, info_ptr);

  png_get_IHDR(png_ptr, info_ptr, &width, &height,
			   &bit_depth, &color_type, &interlace_type, NULL, NULL);

  /* expand data to 24 bit RGB */
  png_set_expand(png_ptr);

  /* convert grayscale to 24 bit RGB */
  png_set_gray_to_rgb(png_ptr);

  /* have the png code handle interlacing */
  png_set_interlace_handling(png_ptr);

  /* add a filler to 24 bit images, so we'll always get 32 bits per pixel */
  png_set_filler(png_ptr, 0, PNG_FILLER_AFTER);

  png_read_update_info(png_ptr, info_ptr);

  image = YCreateImage(display,
					   width, height,
					   YGetColorRGB(0, 0, 0));

  {
	png_bytep *row_pointers;
	int i;

	row_pointers = (png_bytep*)malloc(sizeof(png_bytep));

	for (i = 0; i < height; i ++)
	  row_pointers[i] = (png_bytep)image->image_data + width * i;

	png_read_image(png_ptr, row_pointers);

	free(row_pointers);
  }

  png_read_end(png_ptr, info_ptr);

  png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);

  fclose(fp);

  return image;
}
#endif
