/* -*- Mode: C; tab-width: 4 -*-
   yinput.c -- file descriptor function foo.
   Created: Chris Toshok <toshok@hungry.com>, 14-Sep-1997
 */
/* 
   Copyright (C) 1996, 1997 The Hungry Programmers.
   
   This file is part of The Y Window System.
   
   Y is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
   
   Y is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.
   
   You should have received a copy of the GNU General Public License
   along with GLD; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307 , USA.
*/

#include "ylib_p.h"

#include <sys/types.h>
#include <sys/time.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

typedef struct y_proc_cons
{
  int fd;
  YInputProc proc;
  void *closure;

  struct y_proc_cons *next;
  struct y_proc_cons *prev;
} y_proc_cons;

static y_proc_cons *procs = NULL;

void
YAddInput(int fd,
		  YInputProc proc,
		  void *closure)
{
  y_proc_cons *new_cons = (y_proc_cons*)calloc(1, sizeof(y_proc_cons));

  new_cons->fd = fd;
  new_cons->proc = proc;
  new_cons->closure = closure;
  new_cons->next = procs;

  procs = new_cons;
}

void
YRemoveInput(int fd,
			 YInputProc proc,
			 void *closure)
{
  y_proc_cons *cons;

  for (cons = procs;
	   cons != NULL;
	   cons = cons->next)
	{
	  if (cons->fd == fd
		  && cons->proc == proc
		  && cons->closure == closure)
		{
		  if (cons->next)
			cons->next->prev = cons->prev;
		  if (cons->prev)
			cons->prev->next = cons->next;

		  free(cons);
		}
	}
}

void
y_dispatchInputForFDSet(fd_set *fds)
{
  y_proc_cons *cons;

  for (cons = procs;
	   cons != NULL;
	   cons = cons->next)
	{
	  if (FD_ISSET(cons->fd, fds))
		(*cons->proc)(cons->closure);
	}
}

void
y_addInputFDsToFDSet(fd_set *fds, int *num_fds)
{
  y_proc_cons *cons;
  int nfds = 0;

  FD_ZERO(fds);
  for (cons = procs;
	   cons != NULL;
	   cons = cons->next)
	{
	  if (!FD_ISSET(cons->fd, fds))
		nfds ++;

	  FD_SET(cons->fd, fds);
	}

  *num_fds = nfds;
}
