/* -*- Mode: C; tab-width: 4 -*-
   ylib_p.h -- Y window system client library private header.
   Created: Chris Toshok <toshok@hungry.com>, 8-May-1997
 */
/* 
   Copyright (C) 1996, 1997 The Hungry Programmers.
   
   This file is part of The Y Window System.
   
   Y is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
   
   Y is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.
   
   You should have received a copy of the GNU General Public License
   along with GLD; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307 , USA.
*/

#ifndef _y_lib_p_h
#define _y_lib_p_h

#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>

#include "ytypes.h"
#include "ylib.h"


typedef struct YWindow_vtable {
  void (*Map)(YWindow *window);
  void (*Unmap)(YWindow *window);
  void (*Move)(YWindow *window, int x, int y);
  void (*Resize)(YWindow *window, int width, int height);
  void (*Raise)(YWindow *window);
  void (*Lower)(YWindow *window);
  void (*Clear)(YWindow *window);
  YGC* (*GetGC)(YWindow *window);
  void (*SetBackground)(YWindow *window, YColor background);
  void (*AddInterest)(YWindow *window);
  void (*NotifyInterested)(YWindow *window, Y_Event *event);
  YRect* (*GetBounds)(YWindow *window);
  int (*GetWindowId)(YWindow *window);
  int (*GetSubwindows)(YWindow *window, YWindow ***sub_windows);
  void (*ReleaseSubwindows)(YWindow *window, YWindow **sub_windows,
							int length);
} YWindow_vtable;


struct YWindow_struct {
  YDisplay *display;

  void *image_data; /* used for the offscreen window data. */

  YWindow_id shared_window; /* the info about this window kept in
							   shared memory */

  YWindow_vtable vtable;
};

typedef struct YDisplay_vtable {
  void (*OpenConnection)(YDisplay *display);
  void (*CloseConnection)(YDisplay *display);
  int (*GetConnectionFD)(YDisplay *display);
  YWindow* (*CreateWindow)(YDisplay *display, YWindow *parent_window,
			   int x, int y, int width, int height, YColor bg_color);
  void (*DestroyWindow)(YDisplay *display, YWindow *window);
  YWindow* (*GetWindowById)(YDisplay *display, YWindow_id id);
  int (*ListFonts)(YDisplay *display, const char **font_specs);
  void (*GetNextEvent)(YDisplay *display, Y_Event *event);
} YDisplay_vtable;

struct YDisplay_struct {
  char *display_name;
  YBool is_local;     /* whether or not the display is local. */
  YWindow *root;

  void *connection_data;

  YDisplay_vtable vtable;
};

typedef struct YGC_vtable {
  void (*CopyFromWin)(YGC *gc, YPoint *src_loc, YRect *size, YColor **buffer);
  void (*CopyToWin)(YGC *gc, YPoint *dest_loc, YRect *size, YColor *buffer);
  void (*CopyFromWinToWin)(YGC *dest_gc,
						   YWindow *source, YPoint *src_loc, YPoint *dest_loc,
						   YRect *size);
  void (*DrawLine)(YGC *gc, YPoint *p1, YPoint *p2);
  void (*DrawAsciiText)(YGC *gc, YPoint *where, char *text);
  void (*DrawPoints)(YGC *gc, YPoint **points, int npoints);
  void (*DrawRect)(YGC *gc, YRect *rect);
  void (*FillRect)(YGC *gc, YRect *rect);
  void (*SetForeground)(YGC *gc, YColor foreground);
  void (*SetBackground)(YGC *gc, YColor background);
  void (*SetFont)(YGC *gc, YFont *font);
  void (*Destroy)(YGC *gc);
} YGC_vtable;

struct YGC_struct 
{
  YDisplay *display;	/* the display this gc is for. */
  YWindow *window;		/* the window this gc is for. */

  YGC_id gc_id;

  YColor foreground;
  YColor background;

  YFont *font;

  void *data;

  YGC_vtable vtable;
};

extern void y_dispatchInputForFDSet(fd_set *fds);
extern void y_addInputFDsToFDSet(fd_set *fds, int *nfds);

void Y_handleEvent(Y_Event *event);

void y_sendEvent(YDisplay *display, Y_Event *event);

#ifdef DEBUG
#define D(x) printf x;
#else
#define D(x)
#endif

#endif /* _y_lib_p_h */
