/* -*- Mode: C; tab-width: 4 -*-
   ynet_p.h -- Network communication stuff. 
   Created: Chris Toshok <toshok@hungry.com>, 20-Sep-1997
*/
/* 
   Copyright (C) 1996, 1997 The Hungry Programmers.
   
   This file is part of The Y Window System.
   
   Y is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
   
   Y is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.
   
   You should have received a copy of the GNU General Public License
   along with GLD; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307 , USA.
*/

#ifndef _y_net_p_h
#define _y_net_p_h

#include "ytypes.h"

enum {
  /* window related requests. */
  GET_ROOT_WIN,
  CREATE_WIN,
  MAP_WIN,
  UNMAP_WIN,
  MOVE_WIN,
  RESIZE_WIN,
  RAISE_WIN,
  LOWER_WIN,
  REPARENT_WIN,
  SET_WIN_BACKGROUND,
  CLEAR_WIN,

  LOAD_FONT,

  /* gc related requests */
  GET_GC,
  DESTROY_GC,
  SET_FOREGROUND,
  SET_BACKGROUND,
  SET_FONT,
  DRAW_LINE,
  DRAW_RECT,
  FILL_RECT,

  COPY_FROM_WIN,
  COPY_TO_WIN,
  COPY_FROM_WIN_TO_WIN,

  DRAW_ASCII_TEXT,
  DRAW_UTF8_TEXT
};

typedef struct {
  uint8 tag;
  /* nothing else needed. */
} YNetGetRootWinReq;
typedef struct {
  uint8 tag;
  YWindow_id win_id;
} YNetGetRootWinRep;

typedef struct {
  uint8 tag;
  YWindow_id parent_id;
  uint32 x;
  uint32 y;
  uint32 width;
  uint32 height;
  YColor bg_color;
} YNetCreateWinReq;
typedef struct {
  uint8 tag;
  YWindow_id win_id;
} YNetCreateWinRep;

typedef struct {
  uint8 tag;
  YWindow_id win_id;
} YNetMapWinReq;

typedef struct {
  uint8 tag;
  YWindow_id win_id;
} YNetUnmapWinReq;

typedef struct {
  uint8 tag;
  YWindow_id win_id;
  uint32 x;
  uint32 y;
} YNetMoveWinReq;

typedef struct {
  uint8 tag;
  YWindow_id win_id;
  uint32 width;
  uint32 height;
} YNetResizeWinReq;

typedef struct {
  uint8 tag;
  YWindow_id win_id;
} YNetRaiseWinReq;

typedef struct {
  uint8 tag;
  YWindow_id win_id;
} YNetLowerWinReq;

typedef struct {
  uint8 tag;
  YWindow_id win_id;
  YWindow_id new_parent_id;
} YNetReparentWinReq;

typedef struct {
  uint8 tag;
  YWindow_id win_id;
  YColor bg_color;
} YNetSetWinBackgroundReq;

typedef struct {
  uint8 tag;
  YWindow_id win_id;
} YNetClearWinReq;

typedef struct {
  uint8 tag;
  /* ... XXX ? */
} YNetLoadFontReq;

typedef struct {
  uint8 tag;
  YWindow_id win_id;
} YNetGetGCReq;
typedef struct {
  uint8 tag;
  YGC_id gc_id;
} YNetGetGCRep;

typedef struct {
  uint8 tag;
  YGC_id gc_id;
} YNetDestroyGCReq;

typedef struct {
  uint8 tag;
  YGC_id gc_id;
  YColor foreground;
} YNetSetForegroundReq;

typedef struct {
  uint8 tag;
  YGC_id gc_id;
  YColor background;
} YNetSetBackgroundReq;

typedef struct {
  uint8 tag;
  YGC_id gc_id;
  YFont_id font_id;
} YNetSetFontReq;

typedef struct {
  uint8 tag;
  YGC_id gc_id;
  uint32 x1;
  uint32 y1;
  uint32 x2;
  uint32 y2;
} YNetDrawLineReq;

typedef struct {
  uint8 tag;
  YGC_id gc_id;
  uint32 left;
  uint32 top;
  uint32 right;
  uint32 bottom;
} YNetDrawRectReq;

typedef struct {
  uint8 tag;
  YGC_id gc_id;
  uint32 left;
  uint32 top;
  uint32 right;
  uint32 bottom;
} YNetFillRectReq;

typedef struct {
  uint8 tag;
  YGC_id gc_id;
  uint32 src_x;
  uint32 src_y;
  uint32 width;
  uint32 height;
} YNetCopyFromWinReq;
typedef struct {
  uint8 tag;
  uint32 buffer_size;	/* in bytes. */
  YColor buffer[1];		/* actually, a YColor array of buffer_size/sizeof(YColor) bytes. */
} YNetCopyFromWinRep;
			
typedef struct {
  uint8 tag;
  YGC_id gc_id;
  uint32 dest_x;
  uint32 dest_y;
  uint32 width;
  uint32 height;
  uint32 buffer_size;	/* in bytes. */
  YColor buffer[1];		/* actually, a YColor array of buffer_size/sizeof(YColor) bytes. */
} YNetCopyToWinReq;

typedef struct {
  uint8 tag;
  YGC_id gc_id;
  YWindow_id src_id;
  uint32 src_x;
  uint32 src_y;
  uint32 dest_x;
  uint32 dest_y;
  uint32 width;
  uint32 height;
} YNetCopyFromWinToWinReq;

typedef struct {
  uint8 tag;
  YGC_id gc_id;
  uint32 x;
  uint32 y;
  int txt_length;
  char txt[1];		/* actually, a resizable buffer, whose length is txt_length */
} YNetDrawAsciiTextReq;

typedef struct {
  uint8 tag;
  YGC_id gc_id;
  uint32 x;
  uint32 y;
  int txt_length;
  char txt[1];		/* actually, a resizable buffer, whose length is txt_length */
} YNetDrawUTF8TextReq;

typedef union {
  uint8 tag;

  YNetGetRootWinReq get_root;

  YNetCreateWinReq create_win;
  YNetMapWinReq map_win;
  YNetUnmapWinReq unmap_win;
  YNetMoveWinReq move_win;
  YNetResizeWinReq resize_win;
  YNetRaiseWinReq raise_win;
  YNetLowerWinReq lower_win;
  YNetReparentWinReq reparent_win;
  YNetSetWinBackgroundReq set_win_background;
  YNetClearWinReq clear_win;

  YNetLoadFontReq load_font;
  YNetGetGCReq get_gc;
  YNetDestroyGCReq destroy_gc;

  YNetSetForegroundReq set_fore;
  YNetSetBackgroundReq set_back;
  YNetSetFontReq set_font;
  YNetDrawLineReq draw_line;
  YNetDrawRectReq draw_rect;
  YNetFillRectReq fill_rect;

  YNetCopyFromWinReq copy_from;
  YNetCopyToWinReq copy_to;
  YNetCopyFromWinToWinReq copy_fromto;

  YNetDrawAsciiTextReq draw_ascii;
  YNetDrawUTF8TextReq draw_utf8;
} YNetRequest;

typedef union {
  uint8 tag;
  YNetGetRootWinRep get_root;
  YNetCreateWinRep create_win;
  YNetGetGCRep get_gc;
  YNetCopyFromWinRep copy_from;
} YNetReply;

#endif /* _y_net_p_h */
