/* -*- Mode: C; tab-width: 4 -*-
   yrect.h -- routines available for rectangles.
   Created: Chris Toshok <toshok@hungry.com>, 18-Sep-1997
 */
/* 
   Copyright (C) 1996, 1997 The Hungry Programmers.
   
   This file is part of The Y Window System.
   
   Y is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
   
   Y is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.
   
   You should have received a copy of the GNU General Public License
   along with GLD; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307 , USA.
*/

#include "ylib_p.h"

#include <stdlib.h>

YBool
YDoRectsIntersect(YRect *r1,
				  YRect *r2)
{
  return !(r1->right <= r2->left ||
		   r1->left >= r2->right ||
		   r1->bottom <= r2->top ||
		   r1->top >= r2->bottom );
}

YBool
YIsPointInRect(YPoint *p, YRect *r)
{
  return (p->x > r->left && p->x < r->right &&
		  p->y > r->top && p->y < r->bottom);
}

#ifndef MAX
#define MAX(a,b) ((a) > (b) ? (a) : (b))
#endif
#ifndef MIN
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#endif

YRect *
YRectIntersect(YRect *r1,
			   YRect *r2)
{
  YRect *new_rect = (YRect*)malloc(sizeof(YRect));

  if (!YDoRectsIntersect(r1, r2))
	return NULL;

  new_rect->left = MAX(r1->left, r2->left);
  new_rect->right = MIN(r1->right, r2->right);
  new_rect->top = MAX(r1->top, r2->top);
  new_rect->bottom = MIN(r1->bottom, r2->bottom);

  return new_rect;
}

#undef MAX
#undef MIN
