/* -*- Mode: C; tab-width: 4 -*-
   ywindow.c -- YWindow methods.
   Created: Chris Toshok <toshok@hungry.com>, 8-May-1997
 */
/* 
   Copyright (C) 1996, 1997 The Hungry Programmers.
   
   This file is part of The Y Window System.
   
   Y is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
   
   Y is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.
   
   You should have received a copy of the GNU General Public License
   along with GLD; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307 , USA.
*/

#include "ylib_p.h"
#include <stdio.h>

YWindow *
YCreateWindow(YDisplay *display, YWindow *parent_window,
	      int x, int y, int width, int height,
	      YColor bg_color)
{
  /*  D(("Inside YCreateWindow\n"));*/
  return (*display->vtable.CreateWindow)(display, parent_window,
										 x, y, width, height,
										 bg_color);
}

void
YDestroyWindow(YWindow *window)
{
  (*window->display->vtable.DestroyWindow)(window->display, window);
}

YRect *
YGetWindowBounds(YWindow *window)
{
  if (window->vtable.GetBounds)
	return (*window->vtable.GetBounds)(window);
  else
	return NULL;
}

void
YMapWindow(YWindow *window)
{
  if (window->vtable.Map)
	(*window->vtable.Map)(window);
}

void
YUnmapWindow(YWindow *window)
{
  if (window->vtable.Unmap)
	(*window->vtable.Unmap)(window);
}

void
YRaiseWindow(YWindow *window)
{
  if (window->vtable.Raise)
	(*window->vtable.Raise)(window);
}

void
YLowerWindow(YWindow *window)
{
  if (window->vtable.Lower)
	(*window->vtable.Lower)(window);
}

void
YResizeWindow(YWindow *window, int width, int height)
{
  if (window->vtable.Resize)
	(*window->vtable.Resize)(window, width, height);
}

void
YMoveWindow(YWindow *window, int x, int y)
{
  if (window->vtable.Move)
	(*window->vtable.Move)(window, x, y);
}

void
YClearWindow(YWindow *window)
{
  if (window->vtable.Clear)
	(*window->vtable.Clear)(window);
}

void
YSetWindowBackground(YWindow *window, YColor background)
{
  if (window->vtable.SetBackground)
	(*window->vtable.SetBackground)(window, background);
}

YWindow_id
YGetWindowId(YWindow *window)
{
  if (window->vtable.GetWindowId)
	return (*window->vtable.GetWindowId)(window);
  else
	return 0;
}

YWindow *
YGetWindowById(YDisplay *display, YWindow_id id)
{
  return (*display->vtable.GetWindowById)(display, id);
}

int
YGetSubwindows(YWindow *window, YWindow ***sub_windows)
{
  if (window->vtable.GetSubwindows)
	return (*window->vtable.GetSubwindows)(window, sub_windows);
  else
	return 0;
}

void
YReleaseSubwindows(YWindow *window, YWindow **sub_windows, int length)
{
  if (window->vtable.ReleaseSubwindows)
	(*window->vtable.ReleaseSubwindows)(window, sub_windows, length);
}

void
YAddInterest(YWindow *window)
{
  if (window->vtable.AddInterest)
	(*window->vtable.AddInterest)(window);
}

void
YNotifyInterested(YWindow *window, Y_Event *event)
{
  if (window->vtable.NotifyInterested)
	(*window->vtable.NotifyInterested)(window, event);
}
