/* -*- Mode: C; tab-width: 4 -*-
   ywinmem.c -- Memory YWindow vtable.
   Created: Chris Toshok <toshok@hungry.com>, 8-May-1997
 */
/* 
   Copyright (C) 1996, 1997 The Hungry Programmers.
   
   This file is part of The Y Window System.
   
   Y is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
   
   Y is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.
   
   You should have received a copy of the GNU General Public License
   along with GLD; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307 , USA.
*/

#include "ylib_p.h"
#include "yshm.h"
#include "ymessages.h"
#include <stdio.h>
#include <stdlib.h>

extern YGC_vtable yMemGC_vtable;

static void MemClearWindow(YWindow *window);
static void MemResizeWindow(YWindow *window, int width, int height);
static void MemSetBackground(YWindow *window, YColor background);
static YGC *MemGetGC(YWindow *window);
static YRect *MemGetBounds(YWindow *window);
static int MemGetWindowId(YWindow *window);

YWindow_vtable yMemWindow_vtable = {
  NULL,
  NULL,
  NULL,
  MemResizeWindow,
  NULL,
  NULL,
  MemClearWindow,
  MemGetGC,
  MemSetBackground,
  NULL,
  NULL,
  MemGetBounds,
  MemGetWindowId,
  NULL,
  NULL
};

static void
MemResizeWindow(YWindow *window, int width, int height)
{
}

static void
MemClearWindow(YWindow *window)
{
}

static void
MemSetBackground(YWindow *window, YColor background)
{
}

static YGC *
MemGetGC(YWindow *window)
{
  YGC *new_gc;
 
  /*  D(("Inside MemGetGC()\n"));*/

  new_gc = (YGC*)calloc(1, sizeof(YGC));

  new_gc->vtable = yMemGC_vtable;

  new_gc->window = window;

  return new_gc;
}

static YRect *
MemGetBounds(YWindow *window)
{
  YSharedWindowInfo *info;
  YRect *rect = (YRect*)calloc(1, sizeof(YRect));
  int id;

  /*  D(("Inside LocalGetBounds()\n"));*/

  info = YSHM_getSharedWindowInfo(window->shared_window);
  id = YSHM_lockSemaphore(info->sem_id);

  *rect = info->bounds;

  YSHM_unlockSemaphore(info->sem_id, id);
  YSHM_releaseSharedWindowInfo(info);
  
  return rect;
}

static int
MemGetWindowId(YWindow *window)
{
  return window->shared_window;
}
