/* -*- Mode: C; tab-width: 4 -*-
   ywinnet.c -- Net YWindow vtable.
   Created: Chris Toshok <toshok@hungry.com>, 8-May-1997
 */
/* 
   Copyright (C) 1996, 1997 The Hungry Programmers.
   
   This file is part of The Y Window System.
   
   Y is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
   
   Y is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.
   
   You should have received a copy of the GNU General Public License
   along with GLD; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307 , USA.
*/

#include "ylib_p.h"
#include "ynet_p.h"

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#if defined(linux) || defined(hpux)
#include <netinet/in.h>
#endif

extern YGC_vtable yNetGC_vtable;

extern int NET_RECV_MSG(YDisplay *display, void *buf);
extern int NET_SEND_MSG(YDisplay *display, YNetRequest *req);

static void NetMapWindow(YWindow *window);
static void NetUnmapWindow(YWindow *window);
static void NetClearWindow(YWindow *window);
static void NetResizeWindow(YWindow *window, int width, int height);
static void NetMoveWindow(YWindow *window, int x, int y);
static void NetRaiseWindow(YWindow *window);
static void NetLowerWindow(YWindow *window);
static YGC *NetGetGC(YWindow *window);
static void NetSetBackground(YWindow *window, YColor background);
static void NetAddInterest(YWindow *window);
static void NetNotifyInterested(YWindow *window, Y_Event *event);
static YRect *NetGetBounds(YWindow *window);
static int NetGetWindowId(YWindow *window);
static int NetGetSubwindows(YWindow *window, YWindow ***subwindows);
static void NetReleaseSubwindows(YWindow *window, YWindow **subwindows, int length);

YWindow_vtable yNetWindow_vtable = {
  NetMapWindow,
  NetUnmapWindow,
  NetMoveWindow,
  NetResizeWindow,
  NetRaiseWindow,
  NetLowerWindow,
  NetClearWindow,
  NetGetGC,
  NetSetBackground,
  NetAddInterest,
  NetNotifyInterested,
  NetGetBounds,
  NetGetWindowId,
  NetGetSubwindows,
  NetReleaseSubwindows
};

static void 
NetMapWindow(YWindow *window)
{
  YNetMapWinReq map_win;

  map_win.tag = MAP_WIN;
  map_win.win_id = htonl(window->shared_window);

  NET_SEND_MSG(window->display,
			   (YNetRequest*)&map_win);
  /* no reply */
}

static void 
NetUnmapWindow(YWindow *window)
{
  YNetUnmapWinReq unmap_win;

  unmap_win.tag = UNMAP_WIN;
  unmap_win.win_id = htonl(window->shared_window);

  NET_SEND_MSG(window->display,
			   (YNetRequest*)&unmap_win);
}

static void 
NetClearWindow(YWindow *window)
{
  YNetClearWinReq clear_win;

  clear_win.tag = CLEAR_WIN;
  clear_win.win_id = htonl(window->shared_window);

  NET_SEND_MSG(window->display,
			   (YNetRequest*)&clear_win);
}

static void 
NetResizeWindow(YWindow *window, 
				int width, 
				int height)
{
  YNetResizeWinReq resize_win;

  resize_win.tag = RESIZE_WIN;
  resize_win.win_id = htonl(window->shared_window);
  resize_win.width = htonl(width);
  resize_win.height = htonl(height);

  NET_SEND_MSG(window->display,
			   (YNetRequest*)&resize_win);
}

static void 
NetMoveWindow(YWindow *window, 
			  int x, 
			  int y)
{
  YNetMoveWinReq move_win;

  move_win.tag = MOVE_WIN;
  move_win.win_id = htonl(window->shared_window);
  move_win.x = htonl(x);
  move_win.y = htonl(y);

  NET_SEND_MSG(window->display,
			   (YNetRequest*)&move_win);
}

static void 
NetRaiseWindow(YWindow *window)
{
  YNetRaiseWinReq raise_win;

  raise_win.tag = RAISE_WIN;
  raise_win.win_id = htonl(window->shared_window);

  NET_SEND_MSG(window->display,
			   (YNetRequest*)&raise_win);
}

static void 
NetLowerWindow(YWindow *window)
{
  YNetLowerWinReq lower_win;

  lower_win.tag = LOWER_WIN;
  lower_win.win_id = htonl(window->shared_window);

  NET_SEND_MSG(window->display,
			   (YNetRequest*)&lower_win);
}

static YGC *
NetGetGC(YWindow *window)
{
  YGC *new_gc;
  YNetGetGCReq gc_req;
  YNetReply gc_rep;

  gc_req.tag = GET_GC;
  gc_req.win_id = htonl(window->shared_window);

  NET_SEND_MSG(window->display,
			   (YNetRequest*)&gc_req);

  NET_RECV_MSG(window->display,
			   &gc_rep);

  assert(gc_rep.tag == GET_GC);
  
  new_gc = (YGC*)calloc(1, sizeof(YGC));
  new_gc->display = window->display;
  new_gc->window = window;
  new_gc->vtable = yNetGC_vtable;
  new_gc->gc_id = ntohl(gc_rep.get_gc.gc_id);

  return new_gc;
}

static void
NetSetBackground(YWindow *window, YColor background)
{
  /* nothing yet. */
}

static void 
NetAddInterest(YWindow *window)
{
}

static void 
NetNotifyInterested(YWindow *window, 
					Y_Event *event)
{
}

static YRect *
NetGetBounds(YWindow *window)
{
  return NULL;
}

static int 
NetGetWindowId(YWindow *window)
{
  return window->shared_window;
}

static int 
NetGetSubwindows(YWindow *window, 
				 YWindow ***subwindows)
{
  return 0;
}

static void 
NetReleaseSubwindows(YWindow *window, 
					 YWindow **subwindows, 
					 int length)
{
}

