/* -*- Mode: C; tab-width: 4 -*-
   main.c -- the initialization of the network server
   Created: Chris Toshok <toshok@hungry.com>, 18-Sep-1997
 */
/* 
   Copyright (C) 1996, 1997 The Hungry Programmers.
   
   This file is part of The Y Window System.
   
   Y is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
   
   Y is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.
   
   You should have received a copy of the GNU General Public License
   along with GLD; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307 , USA.
*/

#include "ytypes.h"
#include "client.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef linux
#include <linux/time.h> /* For FD_* macros */
#include <sys/time.h> /* for select */
#endif

static int main_sock; /* the main listening socket. */
YDisplay *_ydisplay;

static void
add_client()
{
  struct sockaddr client_addr;
  int client_addrlen = sizeof(struct sockaddr_in);
  int new_client_fd;

  new_client_fd = accept(main_sock,
						 &client_addr,
						 &client_addrlen);

  if (new_client_fd < 0)
	{
	  perror("add_client(accept)");
	  abort();
	}

  ynet_addClient(new_client_fd);
}

static void
main_loop()
{
  fd_set readfds;

  while (1)
	{
	  FD_ZERO(&readfds);
	  
	  ynet_addClientsToFDSet(&readfds);

	  FD_SET(main_sock, &readfds);

	  select(256, &readfds, NULL, NULL, NULL);
	  
	  if (FD_ISSET(main_sock, &readfds))
		add_client();
	  
	  ynet_handleClientInputFromFDs(&readfds);
	}
}

static void
init_socket()
{
  struct sockaddr_in serv_addr;
  int servlen;

  main_sock = socket(AF_INET, SOCK_STREAM, 0);

  if (main_sock < 0)
	{
	  perror("init_socket(socket)");
	  abort();
	}

  memset((char*)&serv_addr, 0, sizeof(serv_addr));
  serv_addr.sin_family = AF_INET;
  serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
  serv_addr.sin_port = htons(6500);
  servlen = sizeof(struct sockaddr_in);

  if (bind(main_sock, (struct sockaddr*)&serv_addr, servlen) < 0)
	{
	  perror("init_socket(bind)");
	  abort();
	}

  listen (main_sock, 10);
}

static void
init_ydisplay()
{
  char *display_name = getenv("YDISPLAY");

  if (!display_name) display_name = ":0";

  _ydisplay = YOpenDisplay(display_name);
}

int
main(int argc,
	 char **argv)
{
  init_socket();

  init_ydisplay();

  main_loop();

  return 0;
}
