/* -*- Mode: C; tab-width: 4 -*- */
#include "ylib.h"

#include <stdio.h>
#include <stdlib.h>

static YDisplay *display;
static YWindow *button_window;
static YImage *button_text;
static YGC *top_shadow_gc;
static YGC *bottom_shadow_gc;
static YGC *background_gc;
static YGC *foreground_gc;

static YColor top_shadow_color;
static YColor bottom_shadow_color;
static YColor background_color;
static YColor foreground_color;
static YColor arm_background_color;
static YBool armed;

static YFont *font;
static char *label = "Hungry Programmers!";

static int x = 100;
static int y = 100;
static int width = 330;
static int height = 100;

static void
expose()
{
  int shadow_thickness = 4;
  int i;
  YPoint p1, p2, where;
  YRect rect;

  for (i = 0; i < shadow_thickness; i ++)
	{
	  /* top line. */
	  p1.x = i;
	  p1.y = i;
	  p2.x = width - i;
	  p2.y = i;

	  YDrawLine(top_shadow_gc, &p1, &p2);

	  /* left line. */
	  p1.x = i;
	  p1.y = i;
	  p2.x = i;
	  p2.y = height - i;
	  YDrawLine(top_shadow_gc, &p1, &p2);

	  /* right line. */
	  p1.x = width - i;
	  p1.y = i;
	  p2.x = width - i;
	  p2.y = height - i;

	  YDrawLine(bottom_shadow_gc, &p1, &p2);

	  /* bottom line */
	  p1.x = i;
	  p1.y = height - i;
	  p2.x = width - i;
	  p2.y = height - i;

	  YDrawLine(bottom_shadow_gc, &p1, &p2);
	}

  rect.left = i;
  rect.top = i;
  rect.right = width - i;
  rect.bottom = height - i;
 
  YFillRectangle(background_gc, &rect);

  where.x = 10;
  where.y = height / 2 - 20;
  
  {
	YPoint src;
	YRect size;
	
	src.x = src.y = 0;
	size.right = width;
	size.bottom = 50;

	YWinBlit(foreground_gc, button_text,
			 &src, &where, &size);
  };
		   
#if 0
  YDrawAsciiText(foreground_gc, &where, label);
#endif
}

static void
arm()
{
  YSetForeground(top_shadow_gc, bottom_shadow_color);
  YSetForeground(bottom_shadow_gc, top_shadow_color);
  YSetForeground(background_gc, arm_background_color);

  armed = Y_TRUE;

  /* now redraw ourself. */
  expose();
}

static void
disarm()
{
  YSetForeground(top_shadow_gc, top_shadow_color);
  YSetForeground(bottom_shadow_gc, bottom_shadow_color);
  YSetForeground(background_gc, background_color);

  armed = Y_FALSE;

  /* now redraw ourself. */
  expose();
}

static void
activate()
{
  disarm();
  /* no activate function or anything.. */

  printf ("activated.\n");
}

static void
init_colors()
{
  foreground_color = YGetColorRGB(0, 0, 0);
  background_color = YGetColorRGB(175, 175, 175);
  arm_background_color = YGetColorRGB(140, 140, 140);
  top_shadow_color = YGetColorRGB(225, 225, 225);
  bottom_shadow_color = YGetColorRGB(100, 100, 100);
}


int
main(int argc,
     char **argv)
{

  if (argc != 2)
	{
	  fprintf (stderr, "usage:  button <font-url>\n");
	  exit(0);
	}

  display = YOpenDisplay(NULL);

  button_window = YCreateWindow(display, YRootWindowOfDisplay(display),
								x, y, width, height, background_color);

  top_shadow_gc = YGetGC(button_window);
  bottom_shadow_gc = YGetGC(button_window);
  background_gc = YGetGC(button_window);
  foreground_gc = YGetGC(button_window);

  init_colors();

  /* cribbed from disarm(). */
  YSetForeground(top_shadow_gc, top_shadow_color);
  YSetForeground(bottom_shadow_gc, bottom_shadow_color);
  YSetForeground(background_gc, background_color);

  YSetForeground(foreground_gc, foreground_color);
  YSetBackground(foreground_gc, background_color);

  armed = Y_FALSE;

  font = YLoadFont(display, argv[1]);
  if (NULL == font)
	{
	  fprintf (stderr, "Invalid font \"%s\" given.", argv[1]);
	  exit(1);
	}
	
  YSetFont(foreground_gc, font);

  {
	YPoint where;
	YGC *text_gc;

	where.x = 0;
	where.y = 25;

	button_text = YCreateImage(display, 330, 50, background_color);
	text_gc = YGetGC(button_text);
	YSetFont(text_gc, font);
	
	YDrawAsciiText(text_gc, &where, label);

	YDestroyGC(text_gc);
  }

  YAddInterest(button_window);

  YMapWindow(button_window);

  while (1)
    {
      Y_Event y_event;

      YGetNextEvent(display, &y_event);

      switch (y_event.any_event.tag)
		{
		case Y_MouseDownEvent:
		  printf ("got Y_MouseDownEvent\n");
		  arm();
		  break;
		case Y_MouseUpEvent:
		  printf ("got Y_MouseUpEvent\n");
		  /* if the point is inside the button. */
		  activate();
		  break;
		case Y_ExposeEvent:
		  expose();
		  break;
		}
	}
}
